Package: JMbdirect
Type: Package
Title: Joint Model for Longitudinal and Multiple Time to Events Data
Version: 0.1.1
Imports: JMbayes2,joineRML,FastJM,rstanarm,dplyr,jmBIG
Authors@R: person(("Atanu"), "Bhattacharjee",
                  email="atanustat@gmail.com",
                  role=c("aut", "cre"))
Maintainer: Atanu Bhattacharjee <atanustat@gmail.com>
Description: Provides model fitting, prediction, and plotting for joint models of longitudinal and multiple time-to-event data, including methods from Rizopoulos (2012) <doi:10.1201/b12208>. Useful for handling complex survival and longitudinal data in clinical research.
License: GPL-3
Encoding: UTF-8
LazyData: true
Author: Atanu Bhattacharjee [aut, cre]
RoxygenNote: 7.3.2
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-22 07:29:55 UTC; root
Repository: CRAN
Date/Publication: 2026-05-22 08:02:03 UTC
Built: R 4.4.3; ; 2026-05-25 00:40:55 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,jmbB)
S3method(plot,jmcsB)
S3method(plot,jmrmlB)
S3method(plot,jmstB)
S3method(predict,jmbB)
S3method(predict,jmcsB)
S3method(predict,jmrmlB)
S3method(predict,jmstB)
S3method(print,jmbB)
S3method(print,jmcsB)
S3method(print,jmrmlB)
S3method(print,jmstB)
export(bootciJMCS)
export(jmbB)
export(jmcsB)
export(jmrmlB)
export(jmstB)
import(JMbayes2)
import(dplyr)
import(grDevices)
import(graphics)
import(jmBIG)
import(joineRML)
import(rstanarm)
importFrom(FastJM,jmcs)
importFrom(FastJM,survfitjmcs)
importFrom(JMbayes2,jm)
importFrom(rstanarm,VarCorr)
importFrom(rstanarm,stan_jm)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)

#' @title Joint model for Bidirectional survival data using \code{joineRML}
#' @description
#' The function fits joint model for survival data with two events. It utilizes the joineRML package for obtaining the model parameter estimates.
#' @param dtlong longitudinal data
#' @param dtsurv survival data with two event status along with event time
#' @param longm longitudinal model e.g. list(serBilir~drug * year,serBilir ~ drug * year)
#' @param survm survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)
#' @param rd random effect component e.g. list(~year|id,~year|id)
#' @param timeVar time variable
#' @param id ID variable
#' @param samplesize samplesize for bigdata
#' @param BIGdata logical argument TRUE or FALSE
#' @return Estimated model parameters of Joint model with bidirectional survival data
#' @import joineRML
#' @export
#' @references
#' Hickey, Graeme L., et al. "joineRML: a joint model and software package for time-to-event and multivariate longitudinal outcomes." BMC medical research methodology 18 (2018): 1-14.
#'
#' Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
#' @examples
#'  \donttest{
#' ##
#' library(JMbayes2)
#' library(joineRML)
#' jmrmlBModel<-jmrmlB(dtlong=new_long2[new_long2$id%in%c(1:80),],
#'                     dtsurv=new_surv2[new_surv2$id%in%c(1:80),],
#'                     longm=list(y~x7+visit,y~x7+visit),survm=list(Surv(time,status)~x1+visit,
#'                     Surv(time_2,status_2)~x1+visit),rd=list(~visit|id,~visit|id),id='id',
#'                     timeVar='visit',samplesize=40,BIGdata=TRUE)
#' jmrmlBModel
#' ##
#' }
#' @author Atanu Bhattacharjee
jmrmlB<-function(dtlong,dtsurv,longm,survm,rd,timeVar,
                 id,samplesize=200,BIGdata=FALSE){
  cl<-match.call()
  if(!id%in%names(dtlong) ){
    stop("\n Longitudinal data must have column 'id' ")
  }
  if(!id%in%names(dtsurv) ){
    stop("\n Survival data must have column 'id' ")
  }
  if(!names(dtlong)[names(dtlong)==id]==names(dtsurv)[names(dtsurv)==id]){
    stop("\n'dtlong' and 'dtsurv' must have same id.")
  }
  if(!timeVar%in%names(dtlong)){
    stop("\n 'timeVar' should be in longitudinal dataset")
  }


  dtlong<-as.data.frame(dtlong)
  dtsurv<-as.data.frame(dtsurv)
  if(names(dtlong)[names(dtlong)==id]=='id'){dtlong<-dtlong}else{
    dtlong<-dtlong; names(dtlong)[names(dtlong)==id]<-'id'}
  if(names(dtsurv)[names(dtsurv)==id]=='id'){dtsurv<-dtsurv}else{
    dtsurv<-dtsurv; names(dtsurv)[names(dtsurv)==id]<-'id'}
  longm1<-longm[[1]];longm2<-longm[[2]]
  survm1<-survm[[1]];survm2<-survm[[2]]
  rd1<-rd[[1]];rd2<-rd[[2]]
  surv_st1<-all.vars(survm1)[[2]];surv_st2<-all.vars(survm2)[[2]]

  all_variable<-Reduce(union,list(all.vars(longm1),all.vars(longm2),all.vars(survm1),all.vars(survm2),all.vars(rd1),all.vars(rd2)))
  if(anyNA(dtlong[all_variable])){
    dtlong<-dtlong[complete.cases(dtlong[all_variable]),]
    dtsurv<-dtsurv[dtsurv$id%in%intersect(dtsurv$id,unique(dtlong$id)),]
  }
  if(is.factor(dtsurv[,surv_st1])){
    if(!is.numeric(levels(dtsurv[,surv_st1]))){
      stop('Use status variable a numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st1]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st1]))==2&sum(levels(dtsurv[,surv_st1]))!=1){
        stop('More than two possible survival status.Use status variable a numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.factor(dtsurv[,surv_st2])){
    if(!is.numeric(levels(dtsurv[,surv_st2]))){
      stop('Use status variable as numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st2]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st2]))==2&sum(levels(dtsurv[,surv_st2]))!=1){
        stop('Use status variable as numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.numeric(dtsurv[,surv_st1])&&length(unique(dtsurv[,surv_st1]))!=2){
    stop('More than two possible survival status.')
  }

  if(BIGdata==FALSE){
    model1<-mjoint(formLongFixed=longm1,formLongRandom=rd1,formSurv=survm1,data=dtlong
                   ,survData=dtsurv,timeVar= timeVar)
    model2<-mjoint(formLongFixed=longm2,formLongRandom=rd2,formSurv=survm2,data=dtlong
                   ,survData=dtsurv,timeVar= timeVar)
  }else{
    model1<-joinRMLBig(dtlong=dtlong,dtsurv = dtsurv,longm=longm1,survm=survm1,
                       rd=rd1,timeVar=timeVar,samplesize=samplesize,id=id)

    model2<-joinRMLBig(dtlong=dtlong,dtsurv = dtsurv,longm=longm2,survm=survm2,
                       rd=rd2,timeVar=timeVar,samplesize=samplesize,id=id)
  }
  result<-list()
  result$model1<-model1
  result$model2<-model2
  result$timeVar<-timeVar
  result$IDvar<-id
  result$BIGdata<-BIGdata
  class(result)<-'jmrmlB'
  result
}

utils::globalVariables(c('mjoint','na.omit'))

#' @title Joint model for Bidirectional survival data using \code{rstanarm}
#' @description
#' The function fits joint model for survival data with two events. It utilizes the rstanarm package for obtaining the model parameter estimates.
#' @param dtlong longitudinal data
#' @param dtsurv survival data with two event status along with event time
#' @param longm longitudinal model e.g. list(serBilir~drug * year+(year|id),serBilir ~ drug * year+(year|id))
#' @param survm survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)
#' @param timeVar time variable
#' @param id ID variable
#' @param nchain number of MCMC chain
#' @param refresh number of refresh sample
#' @param samplesize samplesize for bigdata
#' @param BIGdata logical argument TRUE or FALSE
#' @return Estimated model parameters of Joint model with bidirectional survival data
#' @importFrom rstanarm stan_jm
#' @import jmBIG
#' @export
#' @references
#' Goodrich, B., et al. "rstanarm: Bayesian applied regression modeling via Stan. R package version 2.17. 4." Online< http://mc-stan. org (2018).
#'
#' Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
#' @examples
#'  \donttest{
#'  ##
#' library(JMbayes2)
#' library(rstanarm)
#' st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,
#'                            new_pbcid$time_2)
#'   new_pbcid
#' }
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:50),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:50),]
#' model_jmstBdirect<-jmstB(
#'   dtlong=pbc2_new,
#'   dtsurv = new_pbc2id,
#'   longm=list(serBilir~drug*year+(year|id),albumin~drug+year+(year|id)),
#'   survm=list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug),
#'   timeVar="year",
#'   id='id',
#'   refresh=400,
#'   nchain=1)
#' model_jmstBdirect
#' ##
#'  }
#' @author Atanu Bhattacharjee
jmstB<-function(dtlong,dtsurv,longm,survm,timeVar,
                id,nchain=1,refresh=1000,BIGdata=FALSE,samplesize=200){

  cl<-match.call()
  if(!id%in%names(dtlong) ){
    stop("\n Longitudinal data must have column 'id' ")
  }
  if(!id%in%names(dtsurv) ){
    stop("\n Survival data must have column 'id' ")
  }
  if(!names(dtlong)[names(dtlong)==id]==names(dtsurv)[names(dtsurv)==id]){
    stop("\n'dtlong' and 'dtsurv' must have same id.")
  }
  if(!timeVar%in%names(dtlong)){
    stop("\n 'timeVar' should be in longitudinal dataset")
  }
  dtlong<-as.data.frame(dtlong)
  dtsurv<-as.data.frame(dtsurv)
  if(names(dtlong)[names(dtlong)==id]=='id'){dtlong<-dtlong}else{
    dtlong<-dtlong; names(dtlong)[names(dtlong)==id]<-'id'}
  if(names(dtsurv)[names(dtsurv)==id]=='id'){dtsurv<-dtsurv}else{
    dtsurv<-dtsurv; names(dtsurv)[names(dtsurv)==id]<-'id'}

  longm1<-longm[[1]];longm2<-longm[[2]]
  survm1<-survm[[1]];survm2<-survm[[2]]
  #rd1<-rd[[1]];rd2<-rd[[2]]
  surv_st1<-all.vars(survm1)[[2]];surv_st2<-all.vars(survm2)[[2]]
  all_variable<-Reduce(union,list(all.vars(longm1),all.vars(longm2),all.vars(survm1),all.vars(survm2)))
  if(anyNA(dtlong[all_variable])){
    dtlong<-dtlong[complete.cases(dtlong[all_variable]),]
    dtsurv<-dtsurv[dtsurv$id%in%intersect(dtsurv$id,unique(dtlong$id)),]
  }
  if(is.factor(dtsurv[,surv_st1])){
    if(!is.numeric(levels(dtsurv[,surv_st1]))){
      stop('Use status variable a numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st1]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st1]))==2&sum(levels(dtsurv[,surv_st1]))!=1){
        stop('More than two possible survival status.Use status variable a numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.factor(dtsurv[,surv_st2])){
    if(!is.numeric(levels(dtsurv[,surv_st2]))){
      stop('Use status variable as numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st2]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st2]))==2&sum(levels(dtsurv[,surv_st2]))!=1){
        stop('Use status variable as numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.numeric(dtsurv[,surv_st1])&&length(unique(dtsurv[,surv_st1]))!=2){
    stop('More than two possible survival status.')
  }

  if(BIGdata==FALSE){
    model1 <- stan_jm(formulaLong =longm1,dataLong = dtlong,formulaEvent = survm1,
                      dataEvent = dtsurv,time_var = timeVar,chains = nchain, refresh = refresh, seed = 12345)

    model2 <- stan_jm(formulaLong =longm2,dataLong = dtlong,formulaEvent = survm2,
                      dataEvent = dtsurv,time_var = timeVar,chains = nchain, refresh = refresh, seed = 12345)
  }else{
    model1<-jmstanBig(dtlong=dtlong,dtsurv =dtsurv,longm=longm1,survm=survm1,
                      samplesize=samplesize,time_var=timeVar,id=id,nchain=nchain,refresh=refresh)
    model2<-jmstanBig(dtlong=dtlong,dtsurv =dtsurv,longm=longm2,survm=survm2,
                      samplesize=samplesize,time_var=timeVar,id=id,nchain=nchain,refresh=refresh)
  }
  result<-list()
  result$model1<-model1
  result$model2<-model2
  result$IDvar<-id
  result$timeVar<-timeVar
  result$BIGdata<-BIGdata
  class(result)<-'jmstB'
  result
}



utils::globalVariables(c('stan_jm','na.omit'))

#' predict.jmstB
#'
#' @param object fitted model
#' @param newdata newdata
#' @param ... others
#' @return Survival Prediction for newdata from model fitted through
#' \code{jmstB()}
#' @import jmBIG
#' @export
#' @method predict jmstB
predict.jmstB<-function(object,newdata,...){
  if(!inherits(object,'jmstB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-object
  model1<-object$model1
  model2<-object$model2
  newdata<-as.data.frame(newdata)
  IDvar<-object$IDvar
  ID<-as.numeric(unique(newdata[,IDvar]))
  timeVar<-object$timeVar

  if(result$BIGdata==FALSE){
    first_long<-all.vars(model1$formula$Long1)[1]
    scnd_long<-all.vars(model2$formula$Long1)[1]
    id_var1<-model1$id_var
    id_var2<-model2$id_var
    long1<-model1$dataLong$Long1
    long2<-model2$dataLong$Long1
    surv1_var<-all.vars(model1$formula$Event)[1]
    surv2_var<-all.vars(model2$formula$Event)[1]
    newdata1<-newdata
    newdata2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    newLdat<-newdata
    newSdat<-newdata[nrow(newdata),]
    plong1<-posterior_traj(model1,ids=ID)
    if(t_0<=t_1){
      psurv1<-posterior_survfit(model1,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20) )
      psurv2<-posterior_survfit(model2,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv2_var,control=list(epoints=20)  )
    }else{
      psurv1<-posterior_survfit(model2,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv2_var,control=list(epoints=20)  )
      psurv2<-posterior_survfit(model1,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20)  )

    }
    plong2<-posterior_traj(model2,ids=ID)
    if(t_0<=t_1){
      psurv1<-psurv1[psurv1$year<=t_1,]
      psurv2<-psurv2[psurv2$year<=2*t_1,]
    }else{
      psurv1<-psurv1[psurv1$year<=t_0,]
      psurv2<-psurv2[psurv2$year<=2*t_0,]
    }
    names(newdata1)[which(names(newdata1)==timeVar)]<-'year'
    names(newdata1)[which(names(newdata1)==first_long)]<-'yvalue1'
    names(newdata2)[which(names(newdata2)==timeVar)]<-'year'
    names(newdata2)[which(names(newdata2)==scnd_long)]<-'yvalue2'
  }else{
    first_long<-all.vars(model1$pseudoMod$formula$Long1)[1]
    scnd_long<-all.vars(model2$pseudoMod$formula$Long1)[1]
    id_var1<-model1$pseudoMod$id_var
    id_var2<-model2$pseudoMod$id_var
    surv1_var<-all.vars(model1$pseudoMod$formula$Event)[1]
    surv2_var<-all.vars(model2$pseudoMod$formula$Event)[1]
    newdata1<-newdata
    newdata2<-newdata
    t_0<-as.numeric(unique(newdata[,surv1_var]))
    t_1<-as.numeric(unique(newdata[,surv2_var]))
    newLdat<-newdata
    newSdat<-newdata[nrow(newdata),]
    plong1<-postTraj(model1,m=1,ids=ID)
    if(t_0<=t_1){
      psurv1<-postSurvfit(model1,ids=ID,extrapolate=TRUE,newdataLong = newLdat,
                          newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20))
      psurv2<-postSurvfit(model2,extrapolate=TRUE,ids=ID,newdataLong=newLdat,
                          newdataEvent=newSdat,last_time=surv2_var,control=list(epoints=20))
    }else{
      psurv1<-postSurvfit(model2,ids=ID,extrapolate=TRUE,newdataLong=newLdat,
                          newdataEvent=newSdat,last_time=surv2_var,control=list(epoints=20))

      psurv2<-postSurvfit(model1,ids=ID,extrapolate=TRUE,newdataLong=newLdat,
                          newdataEvent=newSdat,last_time=surv1_var,control=list(epoints=20))

    }
    plong2<-postTraj(model2,m=1,ids=ID)
    plong1<-plong1$P1
    plong2<-plong2$P1
    psurv1<-psurv1$P1
    names(psurv1)[which(names(psurv1)==timeVar)]<-'year'
    psurv2<-psurv2$P1
    names(psurv2)[which(names(psurv2)==timeVar)]<-'year'
    if(t_0<=t_1){
      psurv1<-psurv1[psurv1$year<=t_1,]
      psurv2<-psurv2[psurv2$year<=2*t_1,]
    }else{
      psurv1<-psurv1[psurv1$year<=t_0,]
      psurv2<-psurv2[psurv2$year<=2*t_0,]
    }
    names(newdata1)[which(names(newdata1)==timeVar)]<-'year'
    names(newdata1)[which(names(newdata1)==first_long)]<-'yvalue1'
    names(newdata2)[which(names(newdata2)==timeVar)]<-'year'
    names(newdata2)[which(names(newdata2)==scnd_long)]<-'yvalue2'

  }
  psurv1<-as.data.frame(psurv1)
  psurv2<-as.data.frame(psurv2)
  predlist<-list()
  predlist$long1<-plong1
  predlist$long2<-plong2
  predlist$surv1<-psurv1
  predlist$surv2<-psurv2
  predlist$ydata<-newdata1
  return(predlist)
}



#' predict.jmcsB
#'
#' @param object fitted model
#' @param newdata newdata
#' @param ... others
#' @return Survival Prediction for newdata from model fitted through
#' \code{jmcsB()}
#' @import jmBIG
#' @export
#' @method predict jmcsB
predict.jmcsB<-function(object,newdata,...){

  if(!inherits(object,'jmcsB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-object
  IDvar<-object$IDvar
  newdata<-as.data.frame(newdata)
  ynewdata<-as.data.frame(newdata)
  cnewdata<-as.data.frame(newdata[!(duplicated(newdata[IDvar])),])
  timeVar<-object$timeVar
  idNumber<-as.numeric(cnewdata[,IDvar])
  if(result$BIGdata==FALSE){
    long1_var<-all.vars(object$model1$LongitudinalSubmodel)[1]
    long2_var<-all.vars(object$model2$LongitudinalSubmodel)[1]
    surv1_var<-all.vars(object$model1$SurvivalSubmodel)[1]
    surv2_var<-all.vars(object$model2$SurvivalSubmodel)[1]
    t_0<-ynewdata[,surv1_var][nrow(ynewdata)]
    t_1<-ynewdata[,surv2_var][nrow(ynewdata)]
    #use bootstrap sample for 95% Confidence Interval

    bootstrap_longitudinal_survival <- function(longitudinal_data, survival_data, n_bootstrap = 10,id,idNumber){
      bootstrap_samples <- vector("list", length = n_bootstrap)
      unique_ids <- as.numeric(unique(longitudinal_data[[id]]))
      for (i in 1:n_bootstrap) {
        # Sample IDs with replacement
        bootstrap_ids <- sample(unique_ids[unique_ids!=idNumber], replace = TRUE)
        n_bootstrap_ids<-seq(1,length(unique(survival_data[[id]])))
        bootstrap_longitudinal_sample <- data.frame()
        bootstrap_survival_sample <- data.frame()
        count<-0
        for (j in 1:(length(bootstrap_ids))) {
          id_longitudinal_data <- longitudinal_data[longitudinal_data[[id]] == bootstrap_ids[j], ]
          id_longitudinal_data[[id]]<-j
          id_survival_data <- survival_data[survival_data[[id]] == bootstrap_ids[[j]], ]
          id_survival_data[[id]]<-j
          bootstrap_longitudinal_sample <- rbind(bootstrap_longitudinal_sample, id_longitudinal_data)
          bootstrap_survival_sample <- rbind(bootstrap_survival_sample, id_survival_data)
        }
        newdatalong<-longitudinal_data[longitudinal_data[id]==idNumber,]
        newdatalong[id]<-length(unique(longitudinal_data[[id]]))
        bootstrap_longitudinal_sample<-rbind(bootstrap_longitudinal_sample,newdatalong)
        newdatasurv<-survival_data[survival_data[id]==idNumber,]
        newdatasurv[id]<-length(unique(longitudinal_data[[id]]))
        bootstrap_survival_sample<-rbind(bootstrap_survival_sample,newdatasurv)
        bootstrap_samples[[i]] <- list(longitudinal = bootstrap_longitudinal_sample, survival = bootstrap_survival_sample)
      }
      return(bootstrap_samples)
    }
    bootstrapped_data <- bootstrap_longitudinal_survival(object$model1$ydata,
                                                         object$model1$cdata,
                                                         n_bootstrap = 30,id=IDvar,
                                                         idNumber = idNumber)

    if(t_0<=t_1){
      future_time1<-seq(t_0,t_1,length.out=20)
      future_time2<-seq(t_1,2*t_1,length.out=20)
      pred_model1<-survfitjmcs(object=result$model1,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
      pred_model2<-survfitjmcs(object=result$model2,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)

      model1_list<-list();model2_list<-list()
      pred_model1_list<-list();pred_model2_list<-list();CIdata_model1<-list();
      CIdata_model2<-list()
      for(i in 1:length(bootstrapped_data)){
        model1_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=object$model1$LongitudinalSubmodel,
                               surv.formula=object$model1$SurvivalSubmodel,
                               random=object$model1$random)
        model2_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=object$model2$LongitudinalSubmodel,
                               surv.formula=object$model2$SurvivalSubmodel,
                               random=object$model2$random)
        pred_model1_list[[i]]<-survfitjmcs(object=model1_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
        pred_model2_list[[i]]<-survfitjmcs(object=model2_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
        CIdata_model1[[i]]<-pred_model1_list[[i]]$Pred[[1]]$PredSurv
        CIdata_model2[[i]]<-pred_model2_list[[i]]$Pred[[1]]$PredSurv
      }

    }else{
      future_time1<-seq(t_1,t_0,length.out=20)
      future_time2<-seq(t_0,2*t_0,length.out=20)
      pred_model1<-survfitjmcs(object=result$model2,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
      pred_model2<-survfitjmcs(object=result$model1,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
      model1_list<-list();model2_list<-list()
      pred_model1_list<-list();pred_model2_list<-list();CIdata_model1<-list()
      CIdata_model2<-list()
      for(i in 1:length(bootstrapped_data)){
        model1_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=object$model2$LongitudinalSubmodel,
                               surv.formula=object$model2$SurvivalSubmodel,
                               random=object$model2$random)
        model2_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=object$model1$LongitudinalSubmodel,
                               surv.formula=object$model1$SurvivalSubmodel,
                               random=object$model1$random)
        pred_model1_list[[i]]<-survfitjmcs(object=model2_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
        pred_model2_list[[i]]<-survfitjmcs(object=model1_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
        CIdata_model1[[i]]<-pred_model1_list[[i]]$Pred[[1]]$PredSurv
        CIdata_model2[[i]]<-pred_model2_list[[i]]$Pred[[1]]$PredSurv

      }
    }
    CIdata_model1<-Reduce('cbind',CIdata_model1)
    CIdata_model2<-Reduce('cbind',CIdata_model2)


    y_obs <- data.frame(year = pred_model1$y.obs[[1]][,timeVar],
                        Marker1 = pred_model1$y.obs[[1]][,long1_var])

    pred_surv <- data.frame(times = pred_model1$Pred[[1]]$times,
                            PredSurv = apply(CIdata_model1,1,function(x){quantile(x,0.5)}),
                            LL=apply(CIdata_model1,1,function(x){quantile(x,0.025)}),
                            UL=apply(CIdata_model1,1,function(x){quantile(x,0.975)})
    )

    y_obs_2<-data.frame(year=pred_model2$y.obs[[1]][,timeVar],
                        Marker2=pred_model2$y.obs[[1]][,long2_var])

    pred_surv2<-data.frame(times = pred_model2$Pred[[1]]$times,
                           PredSurv =apply(CIdata_model2,1,function(x){quantile(x,0.5)}),
                           LL=apply(CIdata_model2,1,function(x){quantile(x,0.025)}),
                           UL=apply(CIdata_model2,1,function(x){quantile(x,0.975)})
    )
  }else{

    long1_var<-all.vars(result$model1$pseudoMod$LongitudinalSubmodel)[1]
    long2_var<-all.vars(result$model2$pseudoMod$LongitudinalSubmodel)[1]
    surv1_var<-all.vars(result$model1$pseudoMod$SurvivalSubmodel)[1]
    surv2_var<-all.vars(result$model2$pseudoMod$SurvivalSubmodel)[1]

    future_time1<-seq(ynewdata[,surv1_var][nrow(ynewdata)],ynewdata[,surv2_var][nrow(ynewdata)],length.out=20)
    future_time2<-seq(ynewdata[,surv2_var][nrow(ynewdata)],2*ynewdata[,surv2_var][nrow(ynewdata)],length.out=20)

    t_0<-ynewdata[,surv1_var][nrow(ynewdata)]
    t_1<-ynewdata[,surv2_var][nrow(ynewdata)]

    if(t_0<=t_1){
      future_time1<-seq(t_0,t_1,length.out=20)
      future_time2<-seq(t_1,2*t_1,length.out=20)
      pred_model1<-survfitJMCS(model=result$model1,ids=idNumber,method='Laplace',u=future_time1,obs.time = timeVar)
      pred_model2<-survfitJMCS(model=result$model2,ids=idNumber,method='Laplace',u=future_time2,obs.time=timeVar)
      bootci_model1<-bootciJMCS(pred_model1,future_time = future_time1)
      bootci_model2<-bootciJMCS(pred_model2,future_time = future_time2)
    }else{
      future_time1<-seq(t_1,t_0,length.out=20)
      future_time2<-seq(t_0,2*t_0,length.out=20)
      pred_model1<-survfitJMCS(model=result$model2,ids=idNumber,method='Laplace',u=future_time1,obs.time = timeVar)
      pred_model2<-survfitJMCS(model=result$model1,ids=idNumber,method='Laplace',u=future_time2,obs.time=timeVar)
      bootci_model1<-bootciJMCS(pred_model1,future_time = future_time1)
      bootci_model2<-bootciJMCS(pred_model2,future_time = future_time2)


    }
    y_obs <- data.frame(year = pred_model1$P1$y.obs[[1]][,timeVar],Marker1 = pred_model1$P1$y.obs[[1]][,long1_var])
    pred_surv <- data.frame(times = bootci_model1$bootCI$Times,
                            PredSurv =bootci_model1$bootCI$Med ,
                            LL=bootci_model1$bootCI$LL,
                            UL=bootci_model1$bootCI$UL)
    y_obs_2<-data.frame(year=pred_model2$P1$y.obs[[1]][,timeVar],
                        Marker2=pred_model2$P1$y.obs[[1]][,long2_var])

    pred_surv2<-data.frame(times = bootci_model2$bootCI$Times,
                           PredSurv =bootci_model2$bootCI$Med,
                           LL=bootci_model2$bootCI$LL,
                           UL=bootci_model2$bootCI$UL)

  }
  pred_surv$PredSurv<-ifelse(pred_surv$PredSurv>=1,1,pred_surv$PredSurv)
  pred_surv$LL<-ifelse(pred_surv$LL>=1,1,pred_surv$LL)
  pred_surv$UL<-ifelse(pred_surv$UL>=1,1,pred_surv$UL)
  pred_surv2$PredSurv<-ifelse(pred_surv2$PredSurv>=1,1,pred_surv2$PredSurv)
  pred_surv2$LL<-ifelse(pred_surv2$LL>=1,1,pred_surv2$LL)
  pred_surv2$UL<-ifelse(pred_surv2$UL>=1,1,pred_surv2$UL)
  predlist<-list()
  predlist$long1<-y_obs
  predlist$surv1<-pred_surv
  predlist$long2<-y_obs_2
  predlist$surv2<-pred_surv2
  predlist$ydata<-ynewdata
  return(predlist)
}


#' predict.jmrmlB
#'
#' @param object fitted model
#' @param newdata newdata
#' @param ... others
#' @return Survival Prediction for newdata from model fitted through \code{jmrmlB()}
#' @import jmBIG
#' @export
#' @method predict jmrmlB
predict.jmrmlB<-function(object,newdata,...){
  if(!inherits(object,'jmrmlB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-object
  model1<-result$model1
  model2<-result$model2
  IDvar<-object$IDvar
  newdata<-as.data.frame(newdata)
   if(result$BIGdata==FALSE){
    longdata1<-model1$data[[1]]
    survdata1<-model1$survData
    longdata2<-model2$data[[1]]
    survdata2<-model2$survData
    surv1_var<-all.vars(object$model1$formSurv)[1]
    surv2_var<-all.vars(object$model2$formSurv)[1]
    long_var1<-all.vars(model1$formLongFixed[[1]])
    time_var1<-model1$timeVar
    long_var2<-all.vars(model2$formLongFixed[[1]])
    time_var2<-model2$timeVar
    names(longdata1)[which(names(longdata1)==model1$id)]<-'id'
    names(longdata2)[which(names(longdata2)==model2$id)]<-'id'
    names(newdata)[which(names(newdata)==IDvar)]<-'id'
    ID<-unique(newdata$id)
    ydata_1<-newdata
    ydata_2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    pred_long1<-data.frame(dynLong(model1,newdata=newdata)[[1]])
    pred_long2<-data.frame(dynLong(model2,newdata=newdata)[[1]])
    if(t_0<=t_1){
      pred_surv1<-dynSurv(model1,newdata=newdata,type='simulated',u=seq(t_0,t_1,length.out=20))
      pred_surv1<-pred_surv1$pred
      pred_surv2<-dynSurv(model2,newdata=newdata,type='simulated',u=seq(t_1,2*t_1,length.out=20))
      pred_surv2<-pred_surv2$pred
    }else{
      pred_surv1<-dynSurv(model2,newdata=newdata,type='simulated',u=seq(t_1,t_0,length.out=20))
      pred_surv1<-pred_surv1$pred
      pred_surv2<-dynSurv(model1,newdata=newdata,type='simulated',u=seq(t_0,2*t_0,length.out=20))
      pred_surv2<-pred_surv2$pred
    }
    names(ydata_1)[which(names(ydata_1)==long_var1[[1]])]<-'y.value1'
    names(ydata_2)[which(names(ydata_2)==long_var2[[1]])]<-'y.value2'
    names(ydata_1)[which(names(ydata_1)==time_var1)]<-'Time'
    names(ydata_2)[which(names(ydata_2)==time_var1)]<-'Time'

  }else{
    longdata1<-list()
    for(i in 1:length(object$model1$allmodel)){
      longdata1[[i]]<-as.data.frame(object$model1$allmodel[[i]]$data)
    }
    longdata1<-Reduce('rbind',longdata1)
    survdata1<-list()
    for(i in 1:length(object$model1$allmodel)){
      survdata1[[i]]<-as.data.frame(object$model1$allmodel[[i]]$survData)
    }
    survdata1<-Reduce('rbind',survdata1)
    longdata2<-list()
    for(i in 1:length(object$model2$allmodel)){
      longdata2[[i]]<-as.data.frame(object$model2$allmodel[[i]]$data)
    }
    longdata2<-Reduce('rbind',longdata2)
    survdata2<-list()
    for(i in 1:length(object$model2$allmodel)){
      survdata2[[i]]<-as.data.frame(object$model2$allmodel[[i]]$survData)
    }
    survdata2<-Reduce('rbind',survdata2)
    surv1_var<-all.vars(object$model1$pseudoMod$formSurv)[1]
    surv2_var<-all.vars(object$model2$pseudoMod$formSurv)[1]
    long_var1<-all.vars(object$model1$pseudoMod$formLongFixed[[1]])[1]
    time_var1<-object$model1$pseudoMod$timeVar
    long_var2<-all.vars(object$model2$pseudoMod$formLongFixed[[1]])[1]
    time_var2<-object$model2$pseudoMod$timeVar
    names(longdata1)[which(names(longdata1)==model1$id)]<-'id'
    names(longdata2)[which(names(longdata2)==model2$id)]<-'id'
    names(newdata)[which(names(newdata)==IDvar)]<-'id'
    ID<-unique(newdata$id)
    ydata_1<-newdata
    ydata_2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    if(t_0<=t_1){
      pred_ls1<-predJRML(model=object$model1,ids=ID,dtlong=longdata1,dtsurv = survdata1,u=seq(t_0,t_1,length.out=20))
      pred_long1<-pred_ls1$plong[[1]]$pred
      pred_surv1<-pred_ls1$psurv[[1]]$pred
      pred_ls2<-predJRML(model=object$model2,ids=ID,dtlong=longdata2,dtsurv = survdata2,u=seq(t_1,2*t_1,length.out=20))
      pred_long2<-pred_ls2$plong[[1]]$pred
      pred_surv2<-pred_ls2$psurv[[1]]$pred
    }else{
      pred_ls1<-predJRML(model=object$model2,ids=ID,dtlong=longdata2,dtsurv = survdata2,u=seq(t_1,t_0,length.out=20))
      pred_long1<-pred_ls1$plong[[1]]$pred
      pred_surv1<-pred_ls1$psurv[[1]]$pred
      pred_ls2<-predJRML(model=object$model1,ids=ID,dtlong=longdata1,dtsurv = survdata1,u=seq(t_0,2*t_0,length.out=20))
      pred_long2<-pred_ls2$plong[[1]]$pred
      pred_surv2<-pred_ls2$psurv[[1]]$pred
    }
    names(ydata_1)[which(names(ydata_1)==long_var1[[1]])]<-'y.value1'
    names(ydata_2)[which(names(ydata_2)==long_var2[[1]])]<-'y.value2'
    names(ydata_1)[which(names(ydata_1)==time_var1)]<-'Time'
    names(ydata_2)[which(names(ydata_2)==time_var1)]<-'Time'
  }

  predlist<-list()
  predlist$long1<-pred_long1
  predlist$long2<-pred_long2
  predlist$surv1<-pred_surv1
  predlist$surv2<-pred_surv2
  return(predlist)
}


#' predict.jmbB
#'
#' @param object fitted model
#' @param newdata newdata
#' @param ... others
#' @return Survival Prediction for newdata from model fitted through \code{jmbB()}
#' @import JMbayes2
#' @import jmBIG
#' @export
#' @method predict jmbB
predict.jmbB<-function(object,newdata,...){
  if(!inherits(object,'jmbB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-object
  ID<-object$IDvar
  newdata<-as.data.frame(newdata)
  t0<-max(newdata[result$timeVar])
  timeVar<-object$timeVar
  idnumber<-as.numeric(unique(newdata[,ID]))
  if(result$BIGdata==FALSE){
    long1_res<-object$model1$model_info$var_names$respVars[[1]]
    long2_res<-object$model2$model_info$var_names$respVars[[1]]
    surv1_var<-object$model1$model_info$var_names$Time_var
    surv2_var<-object$model2$model_info$var_names$Time_var
    surv1_time<-unique(newdata[,surv1_var])
    surv2_time<-unique(newdata[,surv2_var])
    survtime<-c(surv1_time,surv2_time)
    if(isTRUE(surv1_time<surv2_time)){
      surv1_time<-survtime[1]
      surv2_time<-survtime[2]
    }else{
      surv1_time<-survtime[2]
      surv2_time<-survtime[1]
    }
    pred_long1<-predict(result$model1,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv1<-predict(result$model1,newdata=newdata,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }else{
      pred_surv1<-predict(result$model2,newdata=newdata,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }
    pred_long2<-predict(result$model2,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv2<-predict(result$model2,newdata=newdata,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }else{
      pred_surv2<-predict(result$model1,newdata=newdata,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }
    pred_list<-list(long1<-data.frame(Prediction=as.vector(unlist(pred_long1$preds)),
                                      UppL=as.vector(unlist(pred_long1$upp)),
                                      LwrL=as.vector(unlist(pred_long1$low)),
                                      Time=newdata[,timeVar],
                                      yvalue=newdata[,long1_res]),
                    long2<-data.frame(Prediction=as.vector(unlist(pred_long2$preds)),
                                      UppL=as.vector(unlist(pred_long2$upp)),
                                      LwrL=as.vector(unlist(pred_long2$low)),
                                      Time=newdata[,timeVar],
                                      yvalue=newdata[,long2_res]),

                    surv1<-data.frame(Prediction=unlist(pred_surv1$pred),UL=unlist(pred_surv1$upp),LL=unlist(pred_surv1$low),Time=unlist(pred_surv1$times)),
                    surv2<-data.frame(Prediction=unlist(pred_surv2$pred),UL=unlist(pred_surv2$upp),LL=unlist(pred_surv2$low),Time=unlist(pred_surv2$times))
    )
    pred_list[[3]]$Prediction<-1-pred_list[[3]]$Prediction
    pred_list[[3]]$UL<-1-pred_list[[3]]$UL
    pred_list[[3]]$LL<-1-pred_list[[3]]$LL
    pred_list[[4]]$Prediction<-1-pred_list[[4]]$Prediction
    pred_list[[4]]$UL<-1-pred_list[[4]]$UL
    pred_list[[4]]$LL<-1-pred_list[[4]]$LL
  }else{
    long1_res<-result$model1$pseudoMod$model_info$var_names$respVars[[1]]
    long2_res<-result$model2$pseudoMod$model_info$var_names$respVars[[1]]
    surv1_var<-result$model1$pseudoMod$model_info$var_names$Time_var[[1]]
    surv2_var<-result$model2$pseudoMod$model_info$var_names$Time_var[[1]]
    surv1_time<-as.numeric(unique(newdata[,surv1_var]))
    surv2_time<-as.numeric(unique(newdata[,surv2_var]))
    survtime<-c(surv1_time,surv2_time)
    if(isTRUE(surv1_time<surv2_time)){
      surv1_time<-survtime[1]
      surv2_time<-survtime[2]
    }else{
      surv1_time<-survtime[2]
      surv2_time<-survtime[1]
    }

    pred_long1<-predJMbayes(model=result$model1,ids=idnumber,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv1<-predJMbayes(model=result$model1,newdata=newdata,ids=idnumber,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }else{
      pred_surv1<-predJMbayes(model=result$model2,newdata=newdata,ids=idnumber,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }
    pred_long2<-predJMbayes(model=result$model2,newdata=newdata,ids=idnumber,process='longitudinal')

    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv2<-predJMbayes(result$model2,newdata=newdata,ids=idnumber,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }else{
      pred_surv2<-predJMbayes(result$model1,newdata=newdata,ids=idnumber,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }
    pred_list<-list(long1<-data.frame(Prediction=as.vector(unlist(pred_long1$P1[,long1_res])),
                                      UppL=as.vector(unlist(pred_long1$P1[,paste0('upp_',long1_res)])),
                                      LwrL=as.vector(unlist(pred_long1$P1[,paste0('low_',long1_res)])),
                                      Time=as.numeric(unlist(newdata[,timeVar])),
                                      yvalue=as.numeric(unlist(newdata[,long1_res]))),
                    long2<-data.frame(Prediction=as.vector(unlist(pred_long2$P1[,long2_res])),
                                      UppL=as.vector(unlist(pred_long2$P1[,paste0('upp_',long2_res)])),
                                      LwrL=as.vector(unlist(pred_long2$P1[,paste0('low_',long2_res)])),
                                      Time=as.numeric(unlist(newdata[,timeVar])),
                                      yvalue=as.numeric(unlist(newdata[,long2_res]))),

                    surv1<-data.frame(Prediction=unlist(pred_surv1$P1[,"pred_CIF"]),UL=unlist(pred_surv1$P1[,'upp_CIF']),LL=unlist(pred_surv1$P1[,'low_CIF']),Time=unlist(pred_surv1$P1[,timeVar])),
                    surv2<-data.frame(Prediction=unlist(pred_surv2$P1[,"pred_CIF"]),UL=unlist(pred_surv2$P1[,'upp_CIF']),LL=unlist(pred_surv2$P1[,'low_CIF']),Time=unlist(pred_surv2$P1[,timeVar]))
    )
    pred_list[[3]]$Prediction<-1-pred_list[[3]]$Prediction
    pred_list[[3]]$UL<-1-pred_list[[3]]$UL
    pred_list[[3]]$LL<-1-pred_list[[3]]$LL
    pred_list[[4]]$Prediction<-1-pred_list[[4]]$Prediction
    pred_list[[4]]$UL<-1-pred_list[[4]]$UL
    pred_list[[4]]$LL<-1-pred_list[[4]]$LL
  }

  predlist<-list()
  predlist$long1<-pred_list[[1]]
  predlist$long2<-pred_list[[2]]
  predlist$surv1<-pred_list[[3]]
  predlist$surv2<-pred_list[[4]]
  predlist$ydata<-newdata
  return(predlist)
}

#' @title  print.
#' @description
#' print method for class 'jmbB'
#'
#' @param x fitted object
#' @param ... others
#'
#' @return prints table containing various parameter estimates,
#'         SE, P- value for both survival and longitudinal submodel,
#'         if the model is bayesian it includes their credible interval too.
#' @importFrom rstanarm VarCorr
#' @export
#' @rdname print.jmbB
#' @method print jmbB
print.jmbB<-function(x,...){
  #x<-object
  if(!inherits(x,'jmbB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-x
  digits<-3
  if(result$BIGdata==FALSE){
    model1<-result$model1
    model2<-result$model2
    cat('\n Joint model using jmBayes2')
    cat("\n ===========================")
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    cat("\n Longitudinal process:\n")
    ldat1<-list()
    ldat1$Estimate<-c(model1$statistics$Mean$betas1,model1$statistics$Mean$sigmas)
    ldat1$SE<-c(model1$statistics$SD$betas1,model1$statistics$SD$sigmas)
    ldat1$Zvalue<-ldat1$Estimate/ldat1$SE
    ldat1<-data.frame(ldat1)
    f1<-function(x){
      return(if((x[3]>0))2*(1-pnorm((x[3]),mean=0,sd=1))else 2*(pnorm((x[3]),mean=0,sd=1)))
    }
    Pvalue1<-apply(ldat1,1,f1)
    ldat1<-cbind(ldat1,Pvalue1)
    row.names(ldat1)[nrow(ldat1)]<-'sigma'
    ldat1<-round(ldat1,digits=digits)
    names(ldat1)[4]<-'Pvalue'
    print(ldat1)
    cat('\n Survival process for event1: \n ')
    sdat1<-list()
    sdat1$Estimate<-c(model1$statistics$Mean$gammas,model1$statistics$Mean$alphas)
    sdat1$SE<-c(result$model1$statistics$SD$gammas,model1$statistics$Mean$alphas)
    sdat1$Zvalue<-sdat1$Estimate/sdat1$SE
    sdat1<-data.frame(sdat1)
    Pvalue_1<-apply(sdat1,1,f1)
    sdat1<-cbind(sdat1,Pvalue_1)
    sdat1<-round(sdat1,digits=digits)
    names(sdat1)[4]<-'Pvalue'
    print(sdat1)
    cat('\n Random effect covariance matrix :\n')
    D_1<-model1$statistics$Mean$D
    print(D_1)

    cat("\n Summary of second joint model with Event 2: ")
    cat("\n -------------------------------------------")
    cat("\n Longitudinal process:\n")
    ldat2<-list()
    ldat2$Estimate<-c(model2$statistics$Mean$betas1,model2$statistics$Mean$sigmas)
    ldat2$SE<-c(model2$statistics$SD$betas1,model2$statistics$SD$sigmas)
    ldat2$Zvalue<-ldat2$Estimate/ldat2$SE
    ldat2<-data.frame(ldat2)
    f1<-function(x){
      return(if((x[3]>0))2*(1-pnorm((x[3]),mean=0,sd=1))else 2*(pnorm((x[3]),mean=0,sd=1)))
    }
    Pvalue3<-apply(ldat2,1,f1)
    ldat2<-cbind(ldat2,Pvalue3)
    row.names(ldat2)[nrow(ldat2)]<-'sigma'
    ldat2<-round(ldat2,digits=digits)
    names(ldat2)[4]<-'Pvalue'
    print(ldat2)

    cat('\n Survival process for event2: \n ')
    sdat2<-list()
    sdat2$Estimate<-c(model2$statistics$Mean$gammas,model2$statistics$Mean$alphas)
    sdat2$SE<-c(result$model2$statistics$SD$gammas,model2$statistics$Mean$alphas)
    sdat2$Zvalue<-sdat2$Estimate/sdat2$SE
    sdat2<-data.frame(sdat2)
    Pvalue_4<-apply(sdat2,1,f1)
    sdat2<-cbind(sdat2,Pvalue_4)
    sdat2<-round(sdat2,digits=digits)
    names(sdat2)[4]<-'Pvalue'
    print(sdat2)
    cat('\n Random effect covariance matrix :\n')
    D_2<-model2$statistics$Mean$D
    print(D_2)
    invisible(x)
  }else{
    model1<-result$model1
    model2<-result$model2
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    print(model1)
    cat("\n Summary of second joint model with Event 2: ")
    cat("\n -------------------------------------------")
    print(model2)
    invisible(x)
  }
}

#' @title  print.
#' @description
#' print method for class 'jmrmlB'
#' @param x fitted object
#' @param ... others
#'
#' @return prints table containing various parameter estimates,
#'         SE, P- value for both survival and longitudinal submodel,
#'         if the model is bayesian it includes their credible interval too.
#' @importFrom rstanarm VarCorr
#' @export
#' @rdname print.jmrmlB
#' @method print jmrmlB
print.jmrmlB<-function(x,...){
  #x<-object
  if(!inherits(x,'jmrmlB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-x
  digits<-3

  if(result$BIGdata==FALSE){
    model1<-result$model1
    model2<-result$model2

    cat("\n  Joint model using joineRML")
    cat("\n ===========================================================")
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    cat("\n Longitudinal process:\n")
    #cat("\n Total number of events in survival data:")

    vc1<-vcov(model1)
    nd1<-sum(1:dim(model1$coefficients$D)[[2]])
    nb1<-(nd1+1):(nd1+length(model1$coefficients$beta)+1)
    ng1<-(nd1+length(model1$coefficients$beta)+1+1):dim(vc1)[[2]]
    ese1<-diag(vc1)
    ldat1<-list()
    ldat1$Estimate<-c(model1$coefficients$beta,model1$coefficients$sigma2)
    ldat1$SE<-c(ese1[nb1])
    ldat1$Zvalue<-ldat1$Estimate/ldat1$SE
    ldat1<-data.frame(ldat1)
    f1<-function(x){
      return(if((x[3]>0))2*(1-pnorm((x[3]),mean=0,sd=1))else 2*(pnorm((x[3]),mean=0,sd=1)))
    }
    Pvalue1<-apply(ldat1,1,f1)
    ldat1<-cbind(ldat1,Pvalue=Pvalue1)
    ldat1<-round(ldat1,digits=digits)
    print(ldat1)
    cat('\n Survival process for event1: \n ')
    sdat1<-list()
    sdat1$Estimate<-model1$coefficients$gamma
    sdat1$SE<-ese1[ng1]
    sdat1$ZValue<-sdat1$Estimate/sdat1$SE
    sdat1<-data.frame(sdat1)
    sPvalue1<-apply(sdat1,1,f1)
    sdat1<-cbind(sdat1,Pvalue=sPvalue1)
    sdat1<-round(sdat1,digits=digits)
    print(sdat1)
    cat('\n Variance Covariance matrix of Random effects:\n')
    rdat1<-model1$coefficients$D
    if(dim(rdat1)[[2]]==2){
      rvar1<-all.vars(model1$formLongRandom[[1]])
      rownames(rdat1)<-c('Intercept',paste0(rvar1[[1]]))
      colnames(rdat1)<-c('Intercept',paste0(rvar1[[1]]))
    }
    rdat1<-round(rdat1,digits=digits)
    print(rdat1)

    cat("\n Summary of second joint model with Event 2:")
    cat("\n --------------------------------------------")
    cat("\n Longitudinal process:\n")
    vc2<-vcov(model2)
    nd2<-sum(1:dim(model2$coefficients$D)[[2]])
    nb2<-(nd2+1):(nd2+length(model2$coefficients$beta)+1)
    ng2<-(nd2+length(model2$coefficients$beta)+1+1):dim(vc2)[[2]]
    ese2<-diag(vc2)
    ldat2<-list()
    ldat2$Estimate<-c(model2$coefficients$beta,model2$coefficients$sigma2)
    ldat2$SE<-c(ese2[nb2])
    ldat2$Zvalue<-ldat2$Estimate/ldat2$SE
    ldat2<-as.data.frame(ldat2)
    Pvalue2<-apply(ldat2,1,f1)
    ldat2<-cbind(ldat2,Pvalue=Pvalue2)
    ldat2<-round(ldat2,digits=digits)
    print(ldat2)
    cat('\n Survival process for event2: \n ')

    sdat2<-list()
    sdat2$Estimate<-model2$coefficients$gamma
    sdat2$SE<-ese2[ng2]
    sdat2$ZValue<-sdat2$Estimate/sdat2$SE
    sdat2<-data.frame(sdat2)
    sPvalue2<-apply(sdat2,1,f1)
    sdat2<-cbind(sdat2,Pvalue=sPvalue2)
    sdat2<-round(sdat2,digits=digits)
    print(sdat2)

    cat('\n Variance Covariance matrix of Random effects:\n')
    rdat2<-model2$coefficients$D
    if(dim(rdat2)[[2]]==2){
      rvar2<-all.vars(model2$formLongRandom[[1]])
      rownames(rdat2)<-c('Intercept',paste0(rvar2[[1]]))
      colnames(rdat2)<-c('Intercept',paste0(rvar2[[1]]))
    }
    rdat2<-round(rdat2,digits=digits)
    print(rdat2)
    invisible(x)
  }else{
    model1<-result$model1
    model2<-result$model2
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    print(model1)
    cat("\n Summary of second joint model with Event 2: ")
    cat("\n -------------------------------------------")
    print(model2)
    invisible(x)
  }
}


#' @title  print.
#' @description
#' print method for class 'jmcsB'
#' @param x fittedobject
#' @param ... others
#' @return prints table containing various parameter estimates,
#'         SE, P- value for both survival and longitudinal submodel,
#'         if the model is bayesian it includes their credible interval too.
#' @importFrom rstanarm VarCorr
#' @export
#' @rdname print.jmcsB
#' @method print jmcsB
print.jmcsB<-function(x,...){
  #x<-object
  if(!inherits(x,'jmcsB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-x
  digits<-3

  if(x$BIGdata==FALSE){
    cat("\n  Joint model using FastJM")
    cat("\n =========================================================")
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    cat("\n Longitudinal process:\n")
    #cat("\n Total number of events in survival data:" ,result$model1$PropEventType[2,2] ,'\n')
    ldat1<-list()
    ldat1$Estimate<-c(result$model1_est$Par_Beta,result$model1$sigma)
    ldat1$SE<-c(result$model1$sebeta,result$model1$sesigma)
    ldat1$Zvalue<-ldat1$Estimate/ldat1$SE
    ldat1<-data.frame(ldat1)
    f1<-function(x){
      return(if((x[3]>0))2*(1-pnorm((x[3]),mean=0,sd=1))else 2*(pnorm((x[3]),mean=0,sd=1)))
    }
    Pvalue1<-apply(ldat1,1,f1)
    ldat1<-cbind(ldat1,Pvalue=Pvalue1)
    row.names(ldat1)[nrow(ldat1)]<-'sigma^2'
    ldat1<-round(ldat1,digits=digits)
    print(ldat1)
    cat('\n Survival process for event1: \n ')
    sdat1<-list()
    sdat1$Estimate<-result$model1$gamma1
    sdat1$SE<-result$model1$segamma1
    sdat1$Zvalue<-sdat1$Estimate/sdat1$SE
    sdat1<-data.frame(sdat1)
    Pvalue_1<-apply(sdat1,1,f1)
    sdat1<-cbind(sdat1,Pvalue=Pvalue_1)
    sdat1<-round(sdat1,digits=digits)
    print(sdat1)

    cat('\n Association parameters :\n')
    adat1<-list()
    adat1$Estimate<-result$model1$nu1
    adat1$SE<-result$model1$senu1
    adat1$Zvalue<-adat1$Estimate/adat1$SE
    adat1<-data.frame(adat1)
    Pvalue_a1<-apply(adat1,1,f1)
    adat1<-cbind(adat1,Pvalue=Pvalue_a1)
    adat1<-round(adat1,digits = digits)
    random1<-all.vars(result$model1$random)

    if (length(result$model1$nu1) == 2) rownames(adat1) <- c("(Intercept)_1", paste0(random1[1], "_1"))

    adat1<-round(adat1,digits=digits)
    print(adat1)

    cat('\n Variance Covariance matrix of Random effects:\n')
    rdat1<-result$model1$Sig
    if(dim(rdat1)[[2]]==2){
      rvar1<-all.vars(result$model1$random)
      rownames(rdat1)<-c('Intercept',paste0(rvar1[[1]]))
      colnames(rdat1)<-c('Intercept',paste0(rvar1[[1]]))
    }
    rdat<-round(rdat1,digits=digits)
    print(rdat1)

    cat("\n Summary of second joint model with Event 2:")
    cat("\n --------------------------------------------")
    cat("\n Longitudinal process:\n")
    #cat("\n Total number of events in survival data:" ,result$model2$PropEventType[2,2],'\n')

    ldat2<-list()
    ldat2$Estimate<-c(result$model2_est$Par_Beta,result$model2$sigma)
    ldat2$SE<-c(result$model2$sebeta,result$model2$sesigma)
    ldat2$Zvalue<-ldat2$Estimate/ldat2$SE
    ldat2<-data.frame(ldat2)
    f1<-function(x){
      return(if((x[3]>0))2*(1-pnorm((x[3]),mean=0,sd=1))else 2*(pnorm((x[3]),mean=0,sd=1)))
    }
    Pvalue2<-apply(ldat2,1,f1)
    ldat2<-cbind(ldat2,Pvalue=Pvalue2)
    row.names(ldat2)[nrow(ldat2)]<-'sigma^2'
    ldat2<-round(ldat2,digits=digits)
    print(ldat2)
    cat('\n Survival process for event2: \n ')
    sdat2<-list()
    sdat2$Estimate<-result$model2$gamma1
    sdat2$SE<-result$model2$segamma1
    sdat2$Zvalue<-sdat2$Estimate/sdat2$SE
    sdat2<-data.frame(sdat2)
    Pvalue_2<-apply(sdat2,1,f1)
    sdat2<-cbind(sdat2,Pvalue=Pvalue_2)
    sdat2<-round(sdat2,digits=digits)
    print(sdat2)


    cat('\n Association parameters :\n')
    adat2<-list()
    adat2$Estimate<-result$model2$nu1
    adat2$SE<-result$model2$senu1
    adat2$Zvalue<-adat2$Estimate/adat2$SE
    adat2<-data.frame(adat2)
    Pvalue_a2<-apply(adat2,1,f1)
    adat2<-cbind(adat2,Pvalue=Pvalue_a2)
    adat2<-round(adat2,digits = digits)
    random2<-all.vars(result$model2$random)

    if (length(result$model2$nu1) == 2) rownames(adat2) <- c("(Intercept)_1", paste0(random2[1], "_1"))

    adat2<-round(adat2,digits=digits)
    print(adat2)

    cat('\n Variance Covariance matrix of Random effects:\n')
    rdat2<-result$model2$Sig
    if(dim(rdat2)[[2]]==2){
      rvar2<-all.vars(result$model2$random)
      rownames(rdat2)<-c('Intercept',paste0(rvar2[[1]]))
      colnames(rdat2)<-c('Intercept',paste0(rvar2[[1]]))
    }
    rdat2<-round(rdat2,digits=digits)
    print(rdat2)
    invisible(x)
  }else{
    cat("\n Summary of second joint model with Event 1:")
    cat("\n --------------------------------------------")
    print(result$model1)
    cat("\n Summary of second joint model with Event 2:")
    cat("\n --------------------------------------------")
    print(result$model2)
    invisible(x)
  }
}


#' @title  print.
#' @description
#' print method for class 'jmstB'
#' @param x fitted object
#' @param ... others
#' @return prints table containing various parameter estimates,
#'         SE, P- value for both survival and longitudinal submodel,
#'         if the model is bayesian it includes their credible interval too.
#' @importFrom rstanarm VarCorr
#' @import dplyr
#' @export
#' @rdname print.jmstB
#' @method print jmstB
print.jmstB<-function(x,...){
  if(!inherits(x,'jmstB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-x
  digits<-3

  model1<-result$model1
  model2<-result$model2
  if(result$BIGdata==FALSE){
  cat('\n  Joint model using rstanarm')
  cat("\n ===========================================================")
  cat("\n Summary of first joint model with Event 1: ")
  cat("\n -------------------------------------------")
  cat("\n Longitudinal process:\n")
  ldat1<-list()

  vname<-rownames(attr(model1$terms$Long1,'factors'))
  vname<-vname[vname!='id']
  vname[1]<-'Intercept'
  ldat1<-model1$stan_summary[starts_with('Long1',vars =rownames(model1$stan_summary)),c(1,3,4,10)]
  ldat1<-cbind(ldat1,Zvalue=ldat1[,1]/ldat1[,2])
  f1<-function(x){
    return(if((x[1]>0))2*(1-pnorm((x[1]/x[2]),mean=0,sd=1))else 2*(pnorm((x[1]/x[2]),mean=0,sd=1)))
  }
  attr(ldat1,'dimnames')[[2]]<-c('Mean','StDev','2.5%','97.5%','Zvalue')
  Pvalue<-apply(ldat1,1,f1)
  ldat1<-cbind(ldat1,Pvalue)
  row.names(ldat1)<-gsub("Long1\\|","",row.names(ldat1))
  #dat<-data.frame(dat,row.names = c(names(x$pseudoMod$statistics$Mean$betas1),'sigma'))
  ldat1<-round(ldat1,digits=digits)
  print(ldat1)

  cat('\n Survival process for event1: \n ')
  sdat1<-list()
  sdat1<-model1$stan_summary[c(starts_with('Event',vars =rownames(model1$stan_summary)),
                               ends_with('etavalue',vars =rownames(model1$stan_summary)))
                             ,c(1,3,4,10)]
  sdat1<-cbind(sdat1,Zvalue=sdat1[,1]/sdat1[,2])
  attr(sdat1,'dimnames')[[2]]<-c('Mean','StDev','2.5%','97.5%','Zvalue')
  sdat1<-cbind(sdat1,Pvalue=apply(sdat1,1,f1))
  row.names(sdat1)<-gsub('Event\\|','',row.names(sdat1))
  sdat1<-round(sdat1,digits=digits)
  print(sdat1)

  cat('\n Random effect covariance matrix :\n')
  D_1<-VarCorr(model1)
  print(D_1)



  cat("\n Summary of second joint model with Event 2: ")
  cat("\n -------------------------------------------")
  cat("\n Longitudinal process:\n")
  ldat2<-list()

  vname<-rownames(attr(model2$terms$Long1,'factors'))
  vname<-vname[vname!='id']
  vname[1]<-'Intercept'
  ldat2<-model2$stan_summary[starts_with('Long1',vars =rownames(model2$stan_summary)),c(1,3,4,10)]
  ldat2<-cbind(ldat2,Zvalue=ldat2[,1]/ldat2[,2])
  f1<-function(x){
    return(if((x[1]>0))2*(1-pnorm((x[1]/x[2]),mean=0,sd=1))else 2*(pnorm((x[1]/x[2]),mean=0,sd=1)))
  }
  attr(ldat2,'dimnames')[[2]]<-c('Mean','StDev','2.5%','97.5%','Zvalue')
  Pvalue<-apply(ldat2,1,f1)
  ldat2<-cbind(ldat2,Pvalue)
  row.names(ldat2)<-gsub("Long1\\|","",row.names(ldat2))
  #dat<-data.frame(dat,row.names = c(names(x$pseudoMod$statistics$Mean$betas1),'sigma'))
  ldat2<-round(ldat2,digits=digits)
  print(ldat2)

  cat('\n Survival process for event2: \n ')
  sdat2<-list()
  sdat2<-model2$stan_summary[c(starts_with('Event',vars =rownames(model2$stan_summary)),
                               ends_with('etavalue',vars =rownames(model2$stan_summary)))
                             ,c(1,3,4,10)]
  sdat2<-cbind(sdat2,Zvalue=sdat2[,1]/sdat2[,2])
  attr(sdat2,'dimnames')[[2]]<-c('Mean','StDev','2.5%','97.5%','Zvalue')
  sdat2<-cbind(sdat2,Pvalue=apply(sdat2,1,f1))
  row.names(sdat2)<-gsub('Event\\|','',row.names(sdat2))
  sdat2<-round(sdat2,digits=digits)
  print(sdat2)

  cat('\n Random effect covariance matrix :\n')
  D_2<-VarCorr(model2)
  print(D_2)
  invisible(x)
  }else{
    cat("\n Summary of first joint model with Event 1: ")
    cat("\n -------------------------------------------")
    print(model1)
    cat("\n Summary of first joint model with Event 2: ")
    cat("\n -------------------------------------------")
    print(model2)
    invisible(x)
  }
}


utils::globalVariables(c('pnorm','starts_with','ends_with','VarCorr','random','vcov'))

#' @title Prediction plot from \code{jmstB()}
#' @param x fitted model object
#' @param y newdata
#' @param ... others
#' @note
#' In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
#' 8.832. Now suppose we want to look at the survival of ID 2 under joint model
#' 1 after time 4 and for joint model 2 after time 9. For that we created the
#' newdata as if the individual is followed till for a time period
#' less than min(4,9).
#' @return Returns prediction plot for the newdata using the model fitted through \code{jmstB()}.
#' @import rstanarm
#' @import jmBIG
#' @importFrom stats median
#' @examples
#'  \donttest{
#' ##
#' library(JMbayes2)
#' library(rstanarm)
#' st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
#'   new_pbcid
#' }
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:50),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:50),]
#' model_jmstBdirect<-jmstB(
#'   dtlong=pbc2_new,
#'   dtsurv = new_pbc2id,
#'   longm=list(serBilir ~ drug * year+(year|id),albumin~drug+year+(year|id)),
#'   survm=list(Surv(years,status2) ~ drug,Surv(time_2,status_2) ~ drug),
#'   timeVar="year",
#'   id='id',
#'   refresh=400,
#'   nchain=1)
#' t0<-4
#' nd<-pbc2[pbc2$id %in% c(2), ]
#' nd<-nd[nd$year<t0,]
#' nd$status2<-0
#' nd$years<-t0
#' nd$time_2<-9
#' nd$status_2<-0
#' plot(x=model_jmstBdirect,y = nd)
#' ##
#' }
#' @rdname plot.jmstB
#' @method plot jmstB
#' @export
plot.jmstB<-function(x,y,...){
  if(!inherits(x,'jmstB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-x
  model1<-x$model1
  model2<-x$model2
  y<-as.data.frame(y)
  newdata<-y
  IDvar<-x$IDvar
  ID<-as.numeric(unique(y[,IDvar]))
  timeVar<-x$timeVar

  if(result$BIGdata==FALSE){
    first_long<-all.vars(model1$formula$Long1)[1]
    scnd_long<-all.vars(model2$formula$Long1)[1]
    id_var1<-model1$id_var
    id_var2<-model2$id_var
    long1<-model1$dataLong$Long1
    long2<-model2$dataLong$Long1
    surv1_var<-all.vars(model1$formula$Event)[1]
    surv2_var<-all.vars(model2$formula$Event)[1]
    newdata1<-newdata
    newdata2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    newLdat<-newdata
    newSdat<-newdata[nrow(newdata),]
    plong1<-posterior_traj(model1,ids=ID)
    if(t_0<=t_1){
      psurv1<-posterior_survfit(model1,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20) )
      psurv2<-posterior_survfit(model2,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv2_var,control=list(epoints=20)  )
    }else{
      psurv1<-posterior_survfit(model2,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv2_var,control=list(epoints=20)  )
      psurv2<-posterior_survfit(model1,extrapolate=TRUE,newdataLong = newLdat,newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20)  )

    }
    plong2<-posterior_traj(model2,ids=ID)
    if(t_0<=t_1){
      psurv1<-psurv1[psurv1$year<=t_1,]
      psurv2<-psurv2[psurv2$year<=2*t_1,]
    }else{
      psurv1<-psurv1[psurv1$year<=t_0,]
      psurv2<-psurv2[psurv2$year<=2*t_0,]
    }
    names(newdata1)[which(names(newdata1)==timeVar)]<-'year'
    names(newdata1)[which(names(newdata1)==first_long)]<-'yvalue1'
    names(newdata2)[which(names(newdata2)==timeVar)]<-'year'
    names(newdata2)[which(names(newdata2)==scnd_long)]<-'yvalue2'
  }else{
    first_long<-all.vars(model1$pseudoMod$formula$Long1)[1]
    scnd_long<-all.vars(model2$pseudoMod$formula$Long1)[1]
    id_var1<-model1$pseudoMod$id_var
    id_var2<-model2$pseudoMod$id_var
    surv1_var<-all.vars(model1$pseudoMod$formula$Event)[1]
    surv2_var<-all.vars(model2$pseudoMod$formula$Event)[1]
    newdata1<-newdata
    newdata2<-newdata
    t_0<-as.numeric(unique(newdata[,surv1_var]))
    t_1<-as.numeric(unique(newdata[,surv2_var]))
    newLdat<-newdata
    newSdat<-newdata[nrow(newdata),]
    plong1<-postTraj(model1,m=1,ids=ID)
    if(t_0<=t_1){
    psurv1<-postSurvfit(model1,ids=ID,extrapolate=TRUE,newdataLong = newLdat,
                        newdataEvent = newSdat,last_time =surv1_var,control=list(epoints=20))
    psurv2<-postSurvfit(model2,extrapolate=TRUE,ids=ID,newdataLong=newLdat,
                        newdataEvent=newSdat,last_time=surv2_var,control=list(epoints=20))
    }else{
    psurv1<-postSurvfit(model2,ids=ID,extrapolate=TRUE,newdataLong=newLdat,
                        newdataEvent=newSdat,last_time=surv2_var,control=list(epoints=20))

    psurv2<-postSurvfit(model1,ids=ID,extrapolate=TRUE,newdataLong=newLdat,
                        newdataEvent=newSdat,last_time=surv1_var,control=list(epoints=20))

    }
    plong2<-postTraj(model2,m=1,ids=ID)
    plong1<-plong1$P1
    plong2<-plong2$P1
    psurv1<-psurv1$P1
    names(psurv1)[which(names(psurv1)==timeVar)]<-'year'
    psurv2<-psurv2$P1
    names(psurv2)[which(names(psurv2)==timeVar)]<-'year'
    if(t_0<=t_1){
      psurv1<-psurv1[psurv1$year<=t_1,]
      psurv2<-psurv2[psurv2$year<=2*t_1,]
    }else{
      psurv1<-psurv1[psurv1$year<=t_0,]
      psurv2<-psurv2[psurv2$year<=2*t_0,]
    }
    names(newdata1)[which(names(newdata1)==timeVar)]<-'year'
    names(newdata1)[which(names(newdata1)==first_long)]<-'yvalue1'
    names(newdata2)[which(names(newdata2)==timeVar)]<-'year'
    names(newdata2)[which(names(newdata2)==scnd_long)]<-'yvalue2'

  }
  oldpar <- par(no.readonly = TRUE)
  K<-2
  m<-cbind(1:K,rep(K+1,K),rep(K+2,K))
  xticks<-pretty(c(newdata1$year,newdata2$year))
  widths<-c(0.3,0.35,0.35)
  layout(m,widths=widths)
  par(mar=c(0,4.5,0,0),oma=c(4,0,3,0),cex.axis=1.1,font.axis=2,font.lab=2,font.main=2)
  plot(x=newdata1$year,y=newdata1$yvalue1,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=newdata1$year,y=newdata1$yvalue1,pch=8,col='red')
  mtext(first_long,line=2.5,side=2,font=2)
  box(lwd=1.3)
  par(mar = c(0, 4.5, 0, 0))
  plot(x=newdata2$year,y=newdata2$yvalue2,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=newdata2$year,y=newdata2$yvalue2,pch=8,col='red')
  #points(x=newdata2$year,y=newdata2$yvalue2,pch=5,col='green')
  mtext(scnd_long,line=2.5,side=2,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  par(mar=c(0,0,0,0))
  xticks<-pretty(c(psurv1$year))
  plot(x=psurv1$year,y=psurv1$survpred,col='black',type='l',las=1,yaxt='n',xaxt='n',xaxs='i',ylab='',ylim=c(0,1))
  polygon(c(psurv1$year,rev(psurv1$year)),c(psurv1$ci_ub,rev(psurv1$ci_lb)),col=rgb(0,0,0.6,0.4),border=NA)
  lines(x=psurv1$year,y=psurv1$survpred,col='blue',lwd=2)
  text(x=median(psurv1$year),y=0.99,labels = ifelse(t_0<=t_1,"Event 1","Event 2"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  par(mar=c(0,0,0,4.5))
  xticks<-pretty(c(psurv2$year))
  plot(x=psurv2$year,y=psurv2$survpred,col='black',type='l',las=1,yaxt='n',xaxt='n',xaxs='i',ylab='',ylim=c(0,1))
  polygon(c(psurv2$year,rev(psurv2$year)),c(psurv2$ci_ub,rev(psurv2$ci_lb)),col=rgb(0.6,0,0.6,0.4),border = NA)
  lines(x=psurv2$year,y=psurv2$survpred,col='red',lwd=2)
  text(x=median(psurv2$year),y=0.99,labels = ifelse(t_0<=t_1,"Event 2","Event 1"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  axis(4,las=1)
  mtext('Plot for Bidirectional survival data', 3,
        line = 1, outer = TRUE, font = 2, cex = 1.3)
  mtext("Time", 1,
        line = 2.5, outer = TRUE,font=2)
  mtext("Survival Prediction", 4,
        line = 2.5,font=2)
  box(lwd=1.3)
  on.exit(par(oldpar))
}


utils::globalVariables(c('yfit','yvalue1','ci_lb','ci_ub','survpred'))

#' @title Prediction plot from \code{jmcsB()}
#' @param x fitted model object
#' @param y newdata longitudinal
#' @param ... other
#' @note
#' In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
#' 8.832. Now suppose we want to look at the survival of ID 2 under joint model
#' 1 after time 4 and for joint model 2 after time 9. For that we created the
#' newdata as if the individual is followed till for a time period
#' less than min(4,9).
#' @return Returns prediction plot for the newdata using the model fitted through \code{jmcsB()}
#' @import jmBIG
#' @importFrom FastJM survfitjmcs
#' @importFrom stats median quantile
#' @examples
#'  \donttest{
#'  library(JMbayes2)
#'  library(FastJM)
#'   st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
#'   new_pbcid}
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:50),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:50),]
#' model_jmcs<-jmcsB(dtlong=pbc2_new,dtsurv = new_pbc2id,
#'                   longm=list(serBilir~drug*year,
#'                              serBilir~drug*year),
#'                   survm=list(Surv(years,status2)~drug,
#'                              Surv(time_2,status_2)~drug+age),
#'                   rd=list(~1|id,~1|id),
#'                   id='id',timeVar='year')
#'
#' t0<-4
#' nd<-pbc2[pbc2$id %in% c(2),]
#' nd<-nd[nd$year<t0,]
#' nd$status2<-0
#' nd$years<-t0
#' nd$time_2<-9
#' nd$status_2<-0
#' plot(x=model_jmcs,y=nd)
#' ##
#'  }
#' @rdname plot.jmcsB
#' @method plot jmcsB
#' @export
plot.jmcsB<-function(x,y,...){
  if(!inherits(x,'jmcsB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-x
  IDvar<-x$IDvar
  y<-as.data.frame(y)
  ynewdata<-as.data.frame(y)
  cnewdata<-as.data.frame(y[!(duplicated(y[IDvar])),])
  timeVar<-x$timeVar
  idNumber<-as.numeric(cnewdata[,IDvar])
  if(result$BIGdata==FALSE){
    long1_var<-all.vars(x$model1$LongitudinalSubmodel)[1]
    long2_var<-all.vars(x$model2$LongitudinalSubmodel)[1]
    surv1_var<-all.vars(x$model1$SurvivalSubmodel)[1]
    surv2_var<-all.vars(x$model2$SurvivalSubmodel)[1]
    #future_time1<-seq(ynewdata[,timeVar][nrow(ynewdata)],3*ynewdata[,timeVar][nrow(ynewdata)],1)
    t_0<-ynewdata[,surv1_var][nrow(ynewdata)]
    t_1<-ynewdata[,surv2_var][nrow(ynewdata)]
    #use bootstrap sample for 95% Confidence Interval

    bootstrap_longitudinal_survival <- function(longitudinal_data, survival_data, n_bootstrap = 10,id,idNumber){
      bootstrap_samples <- vector("list", length = n_bootstrap)
      unique_ids <- as.numeric(unique(longitudinal_data[[id]]))
      for (i in 1:n_bootstrap) {
        # Sample IDs with replacement
        bootstrap_ids <- sample(unique_ids[unique_ids!=idNumber], replace = TRUE)
        n_bootstrap_ids<-seq(1,length(unique(survival_data[[id]])))
        bootstrap_longitudinal_sample <- data.frame()
        bootstrap_survival_sample <- data.frame()
        count<-0
        for (j in 1:(length(bootstrap_ids))) {
          id_longitudinal_data <- longitudinal_data[longitudinal_data[[id]] == bootstrap_ids[j], ]
          id_longitudinal_data[[id]]<-j
          id_survival_data <- survival_data[survival_data[[id]] == bootstrap_ids[[j]], ]
          id_survival_data[[id]]<-j
          bootstrap_longitudinal_sample <- rbind(bootstrap_longitudinal_sample, id_longitudinal_data)
          bootstrap_survival_sample <- rbind(bootstrap_survival_sample, id_survival_data)
        }
        newdatalong<-longitudinal_data[longitudinal_data[id]==idNumber,]
        newdatalong[id]<-length(unique(longitudinal_data[[id]]))
        bootstrap_longitudinal_sample<-rbind(bootstrap_longitudinal_sample,newdatalong)
        newdatasurv<-survival_data[survival_data[id]==idNumber,]
        newdatasurv[id]<-length(unique(longitudinal_data[[id]]))
        bootstrap_survival_sample<-rbind(bootstrap_survival_sample,newdatasurv)
        bootstrap_samples[[i]] <- list(longitudinal = bootstrap_longitudinal_sample, survival = bootstrap_survival_sample)
      }
      return(bootstrap_samples)
    }
    bootstrapped_data <- bootstrap_longitudinal_survival(x$model1$ydata,
                                                         x$model1$cdata,
                                                         n_bootstrap = 30,id=IDvar,
                                                         idNumber = idNumber)

    if(t_0<=t_1){
      future_time1<-seq(t_0,t_1,length.out=20)
      future_time2<-seq(t_1,2*t_1,length.out=20)
      pred_model1<-survfitjmcs(object=result$model1,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
      pred_model2<-survfitjmcs(object=result$model2,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)

      model1_list<-list();model2_list<-list()
      pred_model1_list<-list();pred_model2_list<-list();CIdata_model1<-list();
      CIdata_model2<-list()
      for(i in 1:length(bootstrapped_data)){
        model1_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=x$model1$LongitudinalSubmodel,
                               surv.formula=x$model1$SurvivalSubmodel,
                               random=x$model1$random)
        model2_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=x$model2$LongitudinalSubmodel,
                               surv.formula=x$model2$SurvivalSubmodel,
                               random=x$model2$random)
        pred_model1_list[[i]]<-survfitjmcs(object=model1_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
        pred_model2_list[[i]]<-survfitjmcs(object=model2_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
        CIdata_model1[[i]]<-pred_model1_list[[i]]$Pred[[1]]$PredSurv
        CIdata_model2[[i]]<-pred_model2_list[[i]]$Pred[[1]]$PredSurv
      }

    }else{
      future_time1<-seq(t_1,t_0,length.out=20)
      future_time2<-seq(t_0,2*t_0,length.out=20)
      pred_model1<-survfitjmcs(object=result$model2,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
      pred_model2<-survfitjmcs(object=result$model1,seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
      model1_list<-list();model2_list<-list()
      pred_model1_list<-list();pred_model2_list<-list();CIdata_model1<-list()
      CIdata_model2<-list()
      for(i in 1:length(bootstrapped_data)){
        model1_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=x$model2$LongitudinalSubmodel,
                               surv.formula=x$model2$SurvivalSubmodel,
                               random=x$model2$random)
        model2_list[[i]]<-jmcs(ydata=bootstrapped_data[[i]]$longitudinal,
                               cdata=bootstrapped_data[[i]]$survival,
                               long.formula=x$model1$LongitudinalSubmodel,
                               surv.formula=x$model1$SurvivalSubmodel,
                               random=x$model1$random)
        pred_model1_list[[i]]<-survfitjmcs(object=model2_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time1,method='Laplace',obs.time=timeVar)
        pred_model2_list[[i]]<-survfitjmcs(object=model1_list[[i]],seed=100,ynewdata=ynewdata,cnewdata=cnewdata,u=future_time2,method='Laplace',obs.time=timeVar)
        CIdata_model1[[i]]<-pred_model1_list[[i]]$Pred[[1]]$PredSurv
        CIdata_model2[[i]]<-pred_model2_list[[i]]$Pred[[1]]$PredSurv

      }
    }
    CIdata_model1<-Reduce('cbind',CIdata_model1)
    CIdata_model2<-Reduce('cbind',CIdata_model2)


    y_obs <- data.frame(year = pred_model1$y.obs[[1]][,timeVar],
                        Marker1 = pred_model1$y.obs[[1]][,long1_var])

    pred_surv <- data.frame(times = pred_model1$Pred[[1]]$times,
                            PredSurv = apply(CIdata_model1,1,function(x){quantile(x,0.5)}),
                            LL=apply(CIdata_model1,1,function(x){quantile(x,0.025)}),
                            UL=apply(CIdata_model1,1,function(x){quantile(x,0.975)})
    )

    y_obs_2<-data.frame(year=pred_model2$y.obs[[1]][,timeVar],
                        Marker2=pred_model2$y.obs[[1]][,long2_var])

    pred_surv2<-data.frame(times = pred_model2$Pred[[1]]$times,
                           PredSurv =apply(CIdata_model2,1,function(x){quantile(x,0.5)}),
                           LL=apply(CIdata_model2,1,function(x){quantile(x,0.025)}),
                           UL=apply(CIdata_model2,1,function(x){quantile(x,0.975)})
    )
    abline_point1<-min(pred_surv$times)
    abline_point2<-max(pred_surv$times)
  }else{

    long1_var<-all.vars(result$model1$pseudoMod$LongitudinalSubmodel)[1]
    long2_var<-all.vars(result$model2$pseudoMod$LongitudinalSubmodel)[1]
    surv1_var<-all.vars(result$model1$pseudoMod$SurvivalSubmodel)[1]
    surv2_var<-all.vars(result$model2$pseudoMod$SurvivalSubmodel)[1]

    future_time1<-seq(ynewdata[,surv1_var][nrow(ynewdata)],ynewdata[,surv2_var][nrow(ynewdata)],length.out=20)
    future_time2<-seq(ynewdata[,surv2_var][nrow(ynewdata)],2*ynewdata[,surv2_var][nrow(ynewdata)],length.out=20)

    t_0<-ynewdata[,surv1_var][nrow(ynewdata)]
    t_1<-ynewdata[,surv2_var][nrow(ynewdata)]

    if(t_0<=t_1){
      future_time1<-seq(t_0,t_1,length.out=20)
      future_time2<-seq(t_1,2*t_1,length.out=20)
      pred_model1<-survfitJMCS(model=result$model1,ids=idNumber,method='Laplace',u=future_time1,obs.time = timeVar)
      pred_model2<-survfitJMCS(model=result$model2,ids=idNumber,method='Laplace',u=future_time2,obs.time=timeVar)
      bootci_model1<-bootciJMCS(pred_model1,future_time = future_time1)
      bootci_model2<-bootciJMCS(pred_model2,future_time = future_time2)
    }else{
      future_time1<-seq(t_1,t_0,length.out=20)
      future_time2<-seq(t_0,2*t_0,length.out=20)
      pred_model1<-survfitJMCS(model=result$model2,ids=idNumber,method='Laplace',u=future_time1,obs.time = timeVar)
      pred_model2<-survfitJMCS(model=result$model1,ids=idNumber,method='Laplace',u=future_time2,obs.time=timeVar)
      bootci_model1<-bootciJMCS(pred_model1,future_time = future_time1)
      bootci_model2<-bootciJMCS(pred_model2,future_time = future_time2)


    }
    y_obs <- data.frame(year = pred_model1$P1$y.obs[[1]][,timeVar],Marker1 = pred_model1$P1$y.obs[[1]][,long1_var])
    pred_surv <- data.frame(times = bootci_model1$bootCI$Times,
                            PredSurv =bootci_model1$bootCI$Med ,
                            LL=bootci_model1$bootCI$LL,
                            UL=bootci_model1$bootCI$UL)
    y_obs_2<-data.frame(year=pred_model2$P1$y.obs[[1]][,timeVar],
                        Marker2=pred_model2$P1$y.obs[[1]][,long2_var])

    pred_surv2<-data.frame(times = bootci_model2$bootCI$Times,
                           PredSurv =bootci_model2$bootCI$Med,
                           LL=bootci_model2$bootCI$LL,
                           UL=bootci_model2$bootCI$UL)

  }
  pred_surv$PredSurv<-ifelse(pred_surv$PredSurv>=1,1,pred_surv$PredSurv)
  pred_surv$LL<-ifelse(pred_surv$LL>=1,1,pred_surv$LL)
  pred_surv$UL<-ifelse(pred_surv$UL>=1,1,pred_surv$UL)
  pred_surv2$PredSurv<-ifelse(pred_surv2$PredSurv>=1,1,pred_surv2$PredSurv)
  pred_surv2$LL<-ifelse(pred_surv2$LL>=1,1,pred_surv2$LL)
  pred_surv2$UL<-ifelse(pred_surv2$UL>=1,1,pred_surv2$UL)
  oldpar <- par(no.readonly = TRUE)
  K<-2
  m<-cbind(1:K,rep(K+1,K),rep(K+2,K))
  xticks<-pretty(c(y_obs$year,y_obs_2$year))
  widths<-c(0.3,0.35,0.35)
  layout(m,widths=widths)
  par(mar=c(0,4.5,0,0),oma=c(4,0,3,0),cex.axis=1.1,font.axis=2,font.lab=2,font.main=2)
  plot(x=y_obs$year,y=y_obs$Marker1,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=y_obs$year,y=y_obs$Marker1,pch=8,col='red')
  mtext(long1_var,line=2.5,side=2,font=2)
  box(lwd=1.3)
  par(mar = c(0, 4.5, 0, 0))
  plot(x=y_obs_2$year,y=y_obs_2$Marker2,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=y_obs_2$year,y=y_obs_2$Marker2,pch=8,col='red')
  mtext(long2_var,line=2.5,side=2,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  par(mar=c(0,0,0,0))
  xticks<-pretty(c(pred_surv$times))
  plot(x=pred_surv$times,y=pred_surv$PredSurv,col='black',type='l',las=1,yaxt='n',xaxs='i',xaxt='n',ylab='',ylim=c(0,1))
  polygon(c(pred_surv$times,rev(pred_surv$times)),c(pred_surv$UL,rev(pred_surv$LL)),col=rgb(0,0,0.6,0.4),border=NA)
  lines(x=pred_surv$times,y=pred_surv$PredSurv,col='blue',lwd=2)
  text(x=median(pred_surv$times),y=0.99,labels = ifelse(t_0<=t_1,"Event 1","Event 2"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  par(mar=c(0,0,0,4.5))
  xticks<-pretty(c(pred_surv2$times))
  plot(x=pred_surv2$times,y=pred_surv2$PredSurv,col='black',type='l',las=1,yaxt='n',xaxt='n',xaxs='i',ylab='',ylim=c(0,1))
  polygon(c(pred_surv2$times,rev(pred_surv2$times)),c(pred_surv2$UL,rev(pred_surv2$LL)),col=rgb(0.6,0,0.6,0.4),border = NA)
  lines(x=pred_surv2$times,y=pred_surv2$PredSurv,col='red',lwd=2)
  text(x=median(pred_surv2$times),y=0.99,labels = ifelse(t_0<=t_1,"Event 2","Event 1"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  axis(4,las=1)
  mtext('Plot for Bidirectional survival data', 3,
        line = 1, outer = TRUE, font = 2, cex = 1.3)
  mtext("Time", 1,
        line = 2.5, outer = TRUE,font=2)
  mtext("Survival Prediction", 4,
        line = 2.5,font=2)
  box(lwd=1.3)
  on.exit(par(oldpar))
}

utils::globalVariables(c('survfitjmcs','year','serBilir','geom_point','times','PredSurv','geom_vline','labs','scale_y_continuous','sec_axis','scale_color_manual','theme','element_text','element_rect'))

#' @title Prediction plot from \code{jmrmlB()}
#' @param x fitted model object
#' @param y newdata
#' @param ... others
#' @note
#' In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
#' 8.832. Now suppose we want to look at the survival of ID 2 under joint model
#' 1 after time 4 and for joint model 2 after time 9. For that we created the
#' newdata as if the individual is followed till for a time period
#' less than min(4,9).
#' @return Returns prediction plot for the newdata using the model fitted through \code{jmrmlB()}.
#' @import joineRML
#' @import grDevices
#' @import graphics
#' @importFrom stats median
#' @export
#' @examples
#'  \donttest{
#' ##
#' library(JMbayes2)
#' library(joineRML)
#' jmrmlBModel<-jmrmlB(dtlong=new_long2[new_long2$id<=400,],
#'                     dtsurv=new_surv2[new_surv2$id<=400,],
#'                     longm=list(y~x7+visit,y~x7+visit),
#'                     survm=list(Surv(time,status)~x1+visit,
#'                                Surv(time_2,status_2)~x1+visit),
#'                     rd=list(~visit|id,~visit|id),
#'                     id='id',
#'                     timeVar='visit',
#'                     samplesize=200,
#'                     BIGdata=TRUE)
#' t0<-6
#' ndBIG<-new_long2[new_long2$id==10,]
#' ndBIG<-ndBIG[ndBIG$visit<t0,]
#' ndBIG$status<-0
#' ndBIG$time<-t0
#' ndBIG$time_2<-10
#' ndBIG$status_2<-0
#' plot(jmrmlBModel,ndBIG)
#' ##
#' }
#' @rdname plot.jmrmlB
#' @method plot jmrmlB
#' @export
plot.jmrmlB<-function(x,y,...){
  if(!inherits(x,'jmrmlB'))
    stop("\n Not a 'JMbdirect' object.\n")
  result<-x
  model1<-result$model1
  model2<-result$model2
  IDvar<-x$IDvar
  y<-as.data.frame(y)
  newdata<-as.data.frame(y)
  if(result$BIGdata==F){
    longdata1<-model1$data[[1]]
    survdata1<-model1$survData
    longdata2<-model2$data[[1]]
    survdata2<-model2$survData
    surv1_var<-all.vars(x$model1$formSurv)[1]
    surv2_var<-all.vars(x$model2$formSurv)[1]
    long_var1<-all.vars(model1$formLongFixed[[1]])
    time_var1<-model1$timeVar
    long_var2<-all.vars(model2$formLongFixed[[1]])
    time_var2<-model2$timeVar
    names(longdata1)[which(names(longdata1)==model1$id)]<-'id'
    names(longdata2)[which(names(longdata2)==model2$id)]<-'id'
    names(newdata)[which(names(newdata)==IDvar)]<-'id'
    ID<-unique(newdata$id)
    ydata_1<-newdata
    ydata_2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    pred_long1<-data.frame(dynLong(model1,newdata=newdata)[[1]])
    pred_long2<-data.frame(dynLong(model2,newdata=newdata)[[1]])
    if(t_0<=t_1){
      pred_surv1<-dynSurv(model1,newdata=newdata,type='simulated',u=seq(t_0,t_1,length.out=20))
      pred_surv1<-pred_surv1$pred
      pred_surv2<-dynSurv(model2,newdata=newdata,type='simulated',u=seq(t_1,2*t_1,length.out=20))
      pred_surv2<-pred_surv2$pred
    }else{
      pred_surv1<-dynSurv(model2,newdata=newdata,type='simulated',u=seq(t_1,t_0,length.out=20))
      pred_surv1<-pred_surv1$pred
      pred_surv2<-dynSurv(model1,newdata=newdata,type='simulated',u=seq(t_0,2*t_0,length.out=20))
      pred_surv2<-pred_surv2$pred
    }
    names(ydata_1)[which(names(ydata_1)==long_var1[[1]])]<-'y.value1'
    names(ydata_2)[which(names(ydata_2)==long_var2[[1]])]<-'y.value2'
    names(ydata_1)[which(names(ydata_1)==time_var1)]<-'Time'
    names(ydata_2)[which(names(ydata_2)==time_var1)]<-'Time'

  }else{
    longdata1<-list()
    for(i in 1:length(x$model1$allmodel)){
      longdata1[[i]]<-as.data.frame(x$model1$allmodel[[i]]$data)
    }
    longdata1<-Reduce('rbind',longdata1)
    survdata1<-list()
    for(i in 1:length(x$model1$allmodel)){
      survdata1[[i]]<-as.data.frame(x$model1$allmodel[[i]]$survData)
    }
    survdata1<-Reduce('rbind',survdata1)
    longdata2<-list()
    for(i in 1:length(x$model2$allmodel)){
      longdata2[[i]]<-as.data.frame(x$model2$allmodel[[i]]$data)
    }
    longdata2<-Reduce('rbind',longdata2)
    survdata2<-list()
    for(i in 1:length(x$model2$allmodel)){
      survdata2[[i]]<-as.data.frame(x$model2$allmodel[[i]]$survData)
    }
    survdata2<-Reduce('rbind',survdata2)
    surv1_var<-all.vars(x$model1$pseudoMod$formSurv)[1]
    surv2_var<-all.vars(x$model2$pseudoMod$formSurv)[1]
    long_var1<-all.vars(x$model1$pseudoMod$formLongFixed[[1]])[1]
    time_var1<-x$model1$pseudoMod$timeVar
    long_var2<-all.vars(x$model2$pseudoMod$formLongFixed[[1]])[1]
    time_var2<-x$model2$pseudoMod$timeVar
    names(longdata1)[which(names(longdata1)==model1$id)]<-'id'
    names(longdata2)[which(names(longdata2)==model2$id)]<-'id'
    names(newdata)[which(names(newdata)==IDvar)]<-'id'
    ID<-unique(newdata$id)
    ydata_1<-newdata
    ydata_2<-newdata
    t_0<-unique(newdata[,surv1_var])
    t_1<-unique(newdata[,surv2_var])
    if(t_0<=t_1){
      pred_ls1<-predJRML(model=x$model1,ids=ID,dtlong=longdata1,dtsurv = survdata1,u=seq(t_0,t_1,length.out=20))
      pred_long1<-pred_ls1$plong[[1]]$pred
      pred_surv1<-pred_ls1$psurv[[1]]$pred
      pred_ls2<-predJRML(model=x$model2,ids=ID,dtlong=longdata2,dtsurv = survdata2,u=seq(t_1,2*t_1,length.out=20))
      pred_long2<-pred_ls2$plong[[1]]$pred
      pred_surv2<-pred_ls2$psurv[[1]]$pred
    }else{
      pred_ls1<-predJRML(model=x$model2,ids=ID,dtlong=longdata2,dtsurv = survdata2,u=seq(t_1,t_0,length.out=20))
      pred_long1<-pred_ls1$plong[[1]]$pred
      pred_surv1<-pred_ls1$psurv[[1]]$pred
      pred_ls2<-predJRML(model=x$model1,ids=ID,dtlong=longdata1,dtsurv = survdata1,u=seq(t_0,2*t_0,length.out=20))
      pred_long2<-pred_ls2$plong[[1]]$pred
      pred_surv2<-pred_ls2$psurv[[1]]$pred
    }
    names(ydata_1)[which(names(ydata_1)==long_var1[[1]])]<-'y.value1'
    names(ydata_2)[which(names(ydata_2)==long_var2[[1]])]<-'y.value2'
    names(ydata_1)[which(names(ydata_1)==time_var1)]<-'Time'
    names(ydata_2)[which(names(ydata_2)==time_var1)]<-'Time'
  }
  oldpar <- par(no.readonly = TRUE)
  K<-2
  m<-cbind(1:K,rep(1+K,K),rep(K+2,K))
  widths<-c(0.3,0.35,0.35)
  layout(m,widths=widths)
  xticks<-pretty(c(ydata_1$Time,ydata_2$Time))
  par(mar=c(0,4.5,0,0),oma=c(4,0,3,0),cex.axis=1.1,font.axis=2,font.lab=2,font.main=2)
  plot(x=ydata_1$Time,y=ydata_1$y.value1,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=ydata_1$Time,y=ydata_1$y.value1,pch=8,col='red')
  mtext(long_var1[1],line=2.5,side=2,font=2)
  box(lwd=1.3)
  par(mar=c(0,4.5,0,0))
  plot(x=ydata_2$Time,y=ydata_2$y.value2,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=ydata_2$Time,y=ydata_2$y.value2,pch=8,col='red')
  mtext(long_var2[1],line=2.5,side=2,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  xticks<-pretty(c(pred_surv1$u))
  par(mar=c(0,0,0,0))
  plot(x=pred_surv1$u,y=pred_surv1$mean,col='black',type='l',las=1,yaxt='n',xaxt='n',xaxs='i',ylab='',ylim=c(0,1),font=2)
  polygon(c(pred_surv1$u,rev(pred_surv1$u)),c(pred_surv1$upper,rev(pred_surv1$lower)),col=rgb(0,0,0.6,0.4),border=NA)
  lines(x=pred_surv1$u,y=pred_surv1$mean,col='blue',lwd=2)
  text(x=median(pred_surv1$u),y=0.99,labels =ifelse(t_0<=t_1,"Event 1","Event 2"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  par(mar=c(0,0,0,4.5))
  xticks<-pretty(c(pred_surv2$u))
  plot(x=pred_surv2$u,y=pred_surv2$mean,col='black',type='l',las=1,yaxt='n',xaxt='n',xaxs='i',ylab='',ylim=c(0,1))
  polygon(c(pred_surv2$u,rev(pred_surv2$u)),c(pred_surv2$upper,rev(pred_surv2$lower)),col=rgb(0.6,0,0.6,0.4),border = NA)
  lines(x=pred_surv2$u,y=pred_surv2$mean,col='red',lwd=2)
  text(x=median(pred_surv2$u),y=0.99,labels =ifelse(t_0<=t_1,"Event 2","Event 1"), pos = 3, col = "black", cex = 1.6,font=2)
  axis(1, at = xticks)
  axis(4,las=1)
  mtext('Plot for Bidirectional survival data', 3,
        line = 1, outer = TRUE, font = 2, cex = 1.3)
  mtext("Time", 1,
        line = 2.5, outer = TRUE,font=2)
  mtext("Survival Prediction", 4,
        line = 2.5,font=2)
  box(lwd=1.3)
  on.exit(par(oldpar))
}

#' @title Joint model for Bidirectional survival data using \code{JMbayes2}
#' @description
#' The function fits joint model for survival data with two events. It utilizes the JMbayes2 package for obtaining the model parameter estimates.
#' @param dtlong longitudinal data
#' @param dtsurv survival data with two event status along with event time
#' @param longm longitudinal model e.g. list(serBilir~drug * year,serBilir ~ drug * year)
#' @param survm survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)
#' @param rd random effect component e.g. list(~year|id,~year|id)
#' @param timeVar time variable
#' @param id ID variable
#' @param niter number if iteration
#' @param nburnin number of sample to burn
#' @param nchain number of MCMC chain
#' @param samplesize samplesize for bigdata
#' @param BIGdata logical argument TRUE or FALSE
#' @return Estimated model parameters of Joint model with bidirectional survival data
#' @importFrom JMbayes2 jm
#' @importFrom stats predict complete.cases
#' @import jmBIG
#' @export
#' @references
#' Rizopoulos, D., G. Papageorgiou, and P. Miranda Afonso. "JMbayes2: extended joint models for longitudinal and time-to-event data." R package version 0.2-4 (2022).
#'
#' Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
#' @examples
#' library(JMbayes2)
#' st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
#'   new_pbcid
#' }
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:100),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:100),]
#' model_jmbBdirect<-jmbB(dtlong=pbc2_new,dtsurv =new_pbc2id,
#'                        longm=list(serBilir~drug*year,serBilir~drug*year),
#'                        survm=list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age),
#'                        rd=list(~year|id,~year|id),
#'                        id='id',timeVar ='year')
#' model_jmbBdirect
#' @author Atanu Bhattacharjee
jmbB<-function(dtlong,dtsurv,longm,survm,rd,timeVar,id,
                 samplesize=NULL,BIGdata=FALSE,niter=200,nburnin=100,nchain=1){
  cl<-match.call()
  if(!id%in%names(dtlong) ){
    stop("\n Longitudinal data must have column 'id' ")
  }
  if(!id%in%names(dtsurv) ){
    stop("\n Survival data must have column 'id' ")
  }
  if(!names(dtlong)[names(dtlong)==id]==names(dtsurv)[names(dtsurv)==id]){
    stop("\n'dtlong' and 'dtsurv' must have same id.")
  }
  if(!timeVar%in%names(dtlong)){
    stop("\n 'timeVar' should be in longitudinal dataset")
  }

  dtlong<-as.data.frame(dtlong)
  dtsurv<-as.data.frame(dtsurv)
  if(names(dtlong)[names(dtlong)==id]=='id'){dtlong<-dtlong}else{
    dtlong<-dtlong; names(dtlong)[names(dtlong)==id]<-'id'}
  if(names(dtsurv)[names(dtsurv)==id]=='id'){dtsurv<-dtsurv}else{
    dtsurv<-dtsurv; names(dtsurv)[names(dtsurv)==id]<-'id'}

  longm1<-longm[[1]];longm2<-longm[[2]]
  survm1<-survm[[1]];survm2<-survm[[2]]
  rd1<-rd[[1]];rd2<-rd[[2]]
  all_variable<-Reduce(union,list(all.vars(longm1),all.vars(longm2),all.vars(survm1),all.vars(survm2),all.vars(rd1),all.vars(rd2)))
  if(anyNA(dtlong[all_variable])){
    dtlong<-dtlong[complete.cases(dtlong[all_variable]),]
    dtsurv<-dtsurv[dtsurv$id%in%intersect(dtsurv$id,unique(dtlong$id)),]
  }
  surv_st1<-all.vars(survm1)[[2]];surv_st2<-all.vars(survm2)[[2]]
  if(is.factor(dtsurv[,surv_st1])){
    if(!is.numeric(levels(dtsurv[,surv_st1]))){
      stop('Use status variable a numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st1]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st1]))==2&sum(levels(dtsurv[,surv_st1]))!=1){
        stop('More than two possible survival status.Use status variable a numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.factor(dtsurv[,surv_st2])){
    if(!is.numeric(levels(dtsurv[,surv_st2]))){
      stop('Use status variable as numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st2]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st2]))==2&sum(levels(dtsurv[,surv_st2]))!=1){
        stop('Use status variable as numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.numeric(dtsurv[,surv_st1])&&length(unique(dtsurv[,surv_st1]))!=2){
    stop('More than two possible survival status.')
  }

  if(BIGdata==FALSE){
    fm1<-lme(fixed=longm1,random=rd1,data=dtlong)
    fm2<-coxph(survm1,data=dtsurv)
    mod1 <- jm(fm2,fm1,time_var=timeVar,n_chains=nchain,n_iter=niter,n_burnin=nburnin)
    fm3<-lme(fixed=longm2,random=rd2,data=dtlong)
    fm4<-coxph(survm2,data=dtsurv)
    mod2 <- jm(fm4,fm3,time_var=timeVar,n_chains=nchain,n_iter=niter,n_burnin=nburnin)
  }else{
    mod1<-jmbayesBig(dtlong=dtlong,dtsurv =dtsurv,longm=longm1,survm=survm1,
                     rd=rd1,timeVar=timeVar,nchain=nchain,samplesize=samplesize,id=id)

    mod2<-jmbayesBig(dtlong=dtlong,dtsurv =dtsurv,longm=longm2,survm=survm2,rd=rd2,
                     timeVar=timeVar,nchain=nchain,samplesize=samplesize,id=id)

  }
  result<-list()
  result$call<-cl
  result$model1<-mod1
  result$model2<-mod2
  result$timeVar<-timeVar
  result$IDvar<-id
  result$BIGdata<-BIGdata
  class(result)<-'jmbB'
  #message('Results for the joint model')
  result
}
utils::globalVariables(c('lme','coxph','jm','na.omit'))

#' @title Joint model for Bidirectional survival data using \code{FastJM}
#' @description
#' The function fits joint model for survival data with two events. It utilizes the FastJM package for obtaining the model parameter estimates.
#' @param dtlong longitudinal data
#' @param dtsurv survival data with two event status along with event time
#' @param longm longitudinal model e.g. list(serBilir~drug * year,serBilir ~ drug * year)
#' @param survm survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)
#' @param rd random effect component e.g. list(~year|id,~year|id)
#' @param id ID variable
#' @param timeVar time variable
#' @param samplesize samplesize for bigdata
#' @param BIGdata logical argument TRUE or FALSE
#' @return Estimated model parameters of Joint model with bidirectional survival data
#' @importFrom FastJM jmcs
#' @import jmBIG
#' @export
#' @references
#'   Li, Shanpeng, et al. "Efficient Algorithms and Implementation of a Semiparametric Joint Model for Longitudinal and Competing Risk Data: With Applications to Massive Biobank Data." Computational and Mathematical Methods in Medicine 2022 (2022).
#'
#'   Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
#' @examples
#' library(FastJM)
#' library(JMbayes2)
#' st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
#'   new_pbcid
#' }
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:50),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:50),]
#' model_jmcs<-jmcsB(dtlong=pbc2_new,dtsurv=new_pbc2id,
#'                   longm=list(serBilir~drug*year,
#'                              serBilir~drug*year),
#'                   survm=list(Surv(years,status2)~drug,
#'                              Surv(time_2,status_2)~drug+age),
#'                   rd=list(~1|id,~1|id),
#'                   id='id',timeVar='year')
#' model_jmcs
#' @author Atanu Bhattacharjee
jmcsB<-function(dtlong,dtsurv,longm,survm,rd,id,timeVar,BIGdata=FALSE,samplesize=200){

  cl<-match.call()

  if(!id%in%names(dtlong) ){
    stop("\n Longitudinal data must have column 'id' ")
  }
  if(!id%in%names(dtsurv) ){
    stop("\n Survival data must have column 'id' ")
  }
  if(!names(dtlong)[names(dtlong)==id]==names(dtsurv)[names(dtsurv)==id]){
    stop("\n'dtlong' and 'dtsurv' must have same id.")
  }
  dtlong<-as.data.frame(dtlong)
  dtsurv<-as.data.frame(dtsurv)
  if(names(dtlong)[names(dtlong)==id]=='id'){dtlong<-dtlong}else{
    dtlong<-dtlong; names(dtlong)[names(dtlong)==id]<-'id'}
  if(names(dtsurv)[names(dtsurv)==id]=='id'){dtsurv<-dtsurv}else{
    dtsurv<-dtsurv; names(dtsurv)[names(dtsurv)==id]<-'id'}
  # Preparing the data
  longm1<-longm[[1]];longm2<-longm[[2]]
  survm1<-survm[[1]];survm2<-survm[[2]]
  rd1<-rd[[1]];rd2<-rd[[2]]
  nr<-nrow(dtsurv)
  surv_st1<-all.vars(survm1)[[2]];surv_st2<-all.vars(survm2)[[2]]
  all_variable<-Reduce(union,list(all.vars(longm1),all.vars(longm2),all.vars(survm1),all.vars(survm2),all.vars(rd1),all.vars(rd2)))
  if(anyNA(dtlong[all_variable])){
    dtlong<-dtlong[complete.cases(dtlong[all_variable]),]
    dtsurv<-dtsurv[dtsurv$id%in%intersect(dtsurv$id,unique(dtlong$id)),]
  }
  if(is.factor(dtsurv[,surv_st1])){
    if(!is.numeric(levels(dtsurv[,surv_st1]))){
      stop('Use status variable a numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st1]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st1]))==2&sum(levels(dtsurv[,surv_st1]))!=1){
        stop('More than two possible survival status.Use status variable a numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.factor(dtsurv[,surv_st2])){
    if(!is.numeric(levels(dtsurv[,surv_st2]))){
      stop('Use status variable as numeric with censored=0 and dead=1')
    }else{

      if(length(levels(dtsurv[,surv_st2]))!=2){
        stop('More than 2 possible values for survival status. Use status variable a numeric with censored=0 and dead=1')
      }

      if(length(levels(dtsurv[,surv_st2]))==2&sum(levels(dtsurv[,surv_st2]))!=1){
        stop('Use status variable as numeric with censored=0 and dead=1')
      }
    }
  }

  if(is.numeric(dtsurv[,surv_st1])&&length(unique(dtsurv[,surv_st1]))!=2){
    stop('More than two possible survival status.')
  }

  if(BIGdata==FALSE){
    model1<-jmcs(long.formula =longm1,ydata = data.frame(dtlong),surv.formula = survm1,
                 cdata = data.frame(dtsurv),random=rd1)
    model2<-jmcs(long.formula =longm2,ydata = data.frame(dtlong),surv.formula = survm2,
                 cdata = data.frame(dtsurv),random=rd2)
  }else{

    model1<-jmcsBig(dtlong=dtlong,dtsurv=dtsurv,longm=longm1,survm=survm1,
                    samplesize=samplesize,rd=rd1,id=id)

    model2<-jmcsBig(dtlong=dtlong,dtsurv=dtsurv,longm=longm2,survm=survm2,
                    rd=rd2,samplesize=samplesize,id=id)

  }

  result<-list()
  result$model1<-model1
  result$model1_est<-list(Par_Beta=model1$beta,Par_Gamma=model1$gamma1,Par_nu=model1$nu1,Par_vcov=model1$vcov)
  result$model2<-model2
  result$model2_est<-list(Par_Beta=model2$beta,Par_Gamma=model2$gamma1,Par_nu=model2$nu1,Par_vcov=model2$vcov)
  result$IDvar<-id
  result$timeVar<-timeVar
  result$BIGdata<-BIGdata
  class(result)<-'jmcsB'
  #samplesize=samplesize# sample of size 50 from the survival data
  #dtsurv1<-split(dtsurv,rep(1:ceiling(nr/samplesize),each=samplesize,length.out=nr))
  return(result)
}


utils::globalVariables(c('jmcs','ID','na.omit'))

#' Function for bootstrapped confidence interval
#'
#' @param object fitted model
#' @param future_time time sequence at which estimates are required
#'
#' @return Returns bootstraped confidence interval for model fitted through FastJM
#' @import jmBIG
#' @export
#'
bootciJMCS<-function(object,future_time){
  if(!inherits(object,"survfitJMCS"))
    stop("\n Not a 'survfitJMCS' object.\n")
  longdata<-object$others$jmcs_others$others$dtlong
  survdata<-object$others$jmcs_others$others$dtsurv
  id<-object$others$jmcs_others$others$id
  idNumber<-object$others$ids
  jmcsModel<-object$others$jmcs_others
  obs.time<-object$others$obs.time
  longm<-object$others$jmcs_others$others$longm
  survm<-object$others$jmcs_others$others$survm
  rd<-object$others$jmcs_others$others$rd
  samplesize<-object$others$jmcs_others$samplesize
  lastid<-max(survdata[[id]])
  bootstrap_longitudinal_survival <- function(longitudinal_data, survival_data, n_bootstrap = 10,id,idNumber){
    bootstrap_samples <- vector("list", length = n_bootstrap)
    unique_ids <- as.numeric(unique(longitudinal_data[[id]]))
    for (i in 1:n_bootstrap) {
      # Sample IDs with replacement
      bootstrap_ids <- sample(unique_ids[unique_ids!=idNumber], replace = TRUE)
      n_bootstrap_ids<-seq(1,length(unique(survival_data[[id]])))
      bootstrap_longitudinal_sample <- data.frame()
      bootstrap_survival_sample <- data.frame()
      count<-0

      for (j in 1:(length(bootstrap_ids))) {
        id_longitudinal_data <- longitudinal_data[longitudinal_data[[id]] == bootstrap_ids[j], ]
        id_longitudinal_data[[id]]<-j
        id_survival_data <- survival_data[survival_data[[id]] == bootstrap_ids[[j]], ]
        id_survival_data[[id]]<-j
        bootstrap_longitudinal_sample <- rbind(bootstrap_longitudinal_sample, id_longitudinal_data)
        bootstrap_survival_sample <- rbind(bootstrap_survival_sample, id_survival_data)
      }
      newdatalong<-longitudinal_data[longitudinal_data[id]==idNumber,]
      newdatalong[id]<-length(unique(longitudinal_data[[id]]))
      bootstrap_longitudinal_sample<-rbind(bootstrap_longitudinal_sample,newdatalong)
      newdatasurv<-survival_data[survival_data[id]==idNumber,]
      newdatasurv[id]<-length(unique(longitudinal_data[[id]]))
      bootstrap_survival_sample<-rbind(bootstrap_survival_sample,newdatasurv)

      bootstrap_samples[[i]] <- list(longitudinal = bootstrap_longitudinal_sample, survival = bootstrap_survival_sample)
    }
    return(bootstrap_samples)
  }
  bootstrapped_data <- bootstrap_longitudinal_survival(longdata,
                                                       survdata,
                                                       n_bootstrap = 10,id=id,
                                                       idNumber = idNumber)

  fit2<-list();P2<-list();P3<-list()
  for(i in 1:10){
    P2<-try(survfitJMCS(model=jmcsModel,ids=idNumber,
                        u=future_time,
                        obs.time=obs.time),silent=TRUE)
    fit2[[i]]<-try(jmcsBig(dtlong=data.frame(bootstrapped_data[[i]]$longitudinal),
                           dtsurv = data.frame(bootstrapped_data[[i]]$survival),
                           longm=longm,survm=survm,
                           rd= rd,samplesize=200,id=id),silent=TRUE)

    P2[[i]]<-try(survfitJMCS(model=fit2[[i]],ids=c(lastid),
                             u=future_time,
                             obs.time=obs.time),silent=TRUE)
    P3[[i]]<-try(P2[[i]]$P1$Pred[[as.character(lastid)]][,2],silent=TRUE)

  }

  CIdata<-Reduce('cbind',P3)
  qCIdata<-data.frame(Times=P2[[i]]$P1$Pred[[as.character(lastid)]]$times,LL=apply(CIdata,1,function(x){quantile(x,0.025)}),Med=apply(CIdata,1,function(x){quantile(x,0.5)}),UL=apply(CIdata,1,function(x){quantile(x,0.975)}))
  result<-list()
  result$lastid<-lastid
  result$bootCI<-qCIdata
  result$P2<-P2
  result$P3<-P3
  class(result)<-"cisurvfitJMCS"
  result

}

#' @title Prediction plot from \code{jmbB()}
#' @param x fitted model
#' @param y newdata
#' @param ... others
#' @note
#' In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
#' 8.832. Now suppose we want to look at the survival of ID 2 under joint model
#' 1 after time 4 and for joint model 2 after time 9. For that we created the
#' newdata as if the individual is followed till for a time period
#' less than min(4,9).
#' @return Returns prediction plot for the newdata using the model fitted through \code{jmbB()}.
#' @import grDevices
#' @import graphics
#' @import JMbayes2
#' @import jmBIG
#' @importFrom stats predict
#' @importFrom stats median
#' @export
#'
#' @examples
#'  \donttest{
#' ##
#' library(JMbayes2)
#' st_pbcid<-function(){
#'   new_pbcid<-pbc2.id
#'   new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
#'   cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
#'   status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
#'   new_pbcid$status_2<-status_2
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
#'   new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
#'   new_pbcid
#' }
#' new_pbc2id<-st_pbcid()
#' pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
#' pbc2_new<-pbc2[pbc2$id%in%c(1:100),]
#' new_pbc2id<-new_pbc2id[new_pbc2id$id%in%c(1:100),]
#' model_jmbBdirect<-jmbB(dtlong=pbc2_new,dtsurv = new_pbc2id,
#'                        longm=list(serBilir ~ drug*year,serBilir ~ drug*year),
#'                        survm=list(Surv(years,status2) ~ drug,
#'                                   Surv(time_2,status_2) ~ drug+age),
#'                        rd=list(~year|id,~year|id),
#'                        id='id',timeVar ='year')
#'
#' t0<-4
#' nd <- pbc2[pbc2$id %in% c(2), ]
#' nd<-nd[nd$year<t0,]
#' nd$status2<-0
#' nd$years<-t0
#' nd$time_2<-9
#' nd$status_2<-0
#' plot(model_jmbBdirect,nd)
#' nd <- pbc2[pbc2$id %in% c(2), ]
#' nd<-nd[nd$year<12,]
#' nd$status2<-0
#' nd$years<-12
#' nd$time_2<-9
#' nd$status_2<-0
#' plot(model_jmbBdirect,nd)
#' ##
#'  }
#' @rdname plot.jmbB
#' @method plot jmbB
#' @export
plot.jmbB<-function(x,y,...){
  if(!inherits(x,'jmbB'))
    stop("\n Not a 'jmbBdirect' object.\n")
  result<-x
  ID<-x$IDvar
  y<-as.data.frame(y)
  newdata<-y
  t0<-max(newdata[result$timeVar])
  timeVar<-x$timeVar
  idnumber<-as.numeric(unique(newdata[,ID]))
  if(result$BIGdata==FALSE){
    long1_res<-x$model1$model_info$var_names$respVars[[1]]
    long2_res<-x$model2$model_info$var_names$respVars[[1]]
    surv1_var<-x$model1$model_info$var_names$Time_var
    surv2_var<-x$model2$model_info$var_names$Time_var
    surv1_time<-unique(newdata[,surv1_var])
    surv2_time<-unique(newdata[,surv2_var])
    survtime<-c(surv1_time,surv2_time)
    if(isTRUE(surv1_time<surv2_time)){
      surv1_time<-survtime[1]
      surv2_time<-survtime[2]
    }else{
      surv1_time<-survtime[2]
      surv2_time<-survtime[1]
    }
    pred_long1<-predict(result$model1,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv1<-predict(result$model1,newdata=newdata,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }else{
      pred_surv1<-predict(result$model2,newdata=newdata,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }
    pred_long2<-predict(result$model2,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv2<-predict(result$model2,newdata=newdata,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }else{
      pred_surv2<-predict(result$model1,newdata=newdata,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }
    pred_list<-list(long1<-data.frame(Prediction=as.vector(unlist(pred_long1$preds)),
                                      UppL=as.vector(unlist(pred_long1$upp)),
                                      LwrL=as.vector(unlist(pred_long1$low)),
                                      Time=newdata[,timeVar],
                                      yvalue=newdata[,long1_res]),
                    long2<-data.frame(Prediction=as.vector(unlist(pred_long2$preds)),
                                      UppL=as.vector(unlist(pred_long2$upp)),
                                      LwrL=as.vector(unlist(pred_long2$low)),
                                      Time=newdata[,timeVar],
                                      yvalue=newdata[,long2_res]),

                    surv1<-data.frame(Prediction=unlist(pred_surv1$pred),UL=unlist(pred_surv1$upp),LL=unlist(pred_surv1$low),Time=unlist(pred_surv1$times)),
                    surv2<-data.frame(Prediction=unlist(pred_surv2$pred),UL=unlist(pred_surv2$upp),LL=unlist(pred_surv2$low),Time=unlist(pred_surv2$times))
    )
    pred_list[[3]]$Prediction<-1-pred_list[[3]]$Prediction
    pred_list[[3]]$UL<-1-pred_list[[3]]$UL
    pred_list[[3]]$LL<-1-pred_list[[3]]$LL
    pred_list[[4]]$Prediction<-1-pred_list[[4]]$Prediction
    pred_list[[4]]$UL<-1-pred_list[[4]]$UL
    pred_list[[4]]$LL<-1-pred_list[[4]]$LL
  }else{
    long1_res<-result$model1$pseudoMod$model_info$var_names$respVars[[1]]
    long2_res<-result$model2$pseudoMod$model_info$var_names$respVars[[1]]
    surv1_var<-result$model1$pseudoMod$model_info$var_names$Time_var[[1]]
    surv2_var<-result$model2$pseudoMod$model_info$var_names$Time_var[[1]]
    surv1_time<-as.numeric(unique(newdata[,surv1_var]))
    surv2_time<-as.numeric(unique(newdata[,surv2_var]))
    survtime<-c(surv1_time,surv2_time)
    if(isTRUE(surv1_time<surv2_time)){
      surv1_time<-survtime[1]
      surv2_time<-survtime[2]
    }else{
      surv1_time<-survtime[2]
      surv2_time<-survtime[1]
    }

    pred_long1<-predJMbayes(model=result$model1,ids=idnumber,newdata=newdata,process='longitudinal')
    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv1<-predJMbayes(model=result$model1,newdata=newdata,ids=idnumber,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }else{
      pred_surv1<-predJMbayes(model=result$model2,newdata=newdata,ids=idnumber,process='event',times=seq(surv1_time,surv2_time,length.out=20))
    }
    pred_long2<-predJMbayes(result$model2,newdata=newdata,ids=idnumber,process='longitudinal')

    if(isTRUE(survtime[1]<survtime[2])){
      pred_surv2<-predJMbayes(result$model2,newdata=newdata,ids=idnumber,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }else{
      pred_surv2<-predJMbayes(result$model1,newdata=newdata,ids=idnumber,process='event',times=seq(surv2_time,2*surv2_time,length.out=20))
    }
    pred_list<-list(long1<-data.frame(Prediction=as.vector(unlist(pred_long1$P1[,long1_res])),
                                      UppL=as.vector(unlist(pred_long1$P1[,paste0('upp_',long1_res)])),
                                      LwrL=as.vector(unlist(pred_long1$P1[,paste0('low_',long1_res)])),
                                      Time=as.numeric(unlist(newdata[,timeVar])),
                                      yvalue=as.numeric(unlist(newdata[,long1_res]))),
                    long2<-data.frame(Prediction=as.vector(unlist(pred_long2$P1[,long2_res])),
                                      UppL=as.vector(unlist(pred_long2$P1[,paste0('upp_',long2_res)])),
                                      LwrL=as.vector(unlist(pred_long2$P1[,paste0('low_',long2_res)])),
                                      Time=as.numeric(unlist(newdata[,timeVar])),
                                      yvalue=as.numeric(unlist(newdata[,long2_res]))),

                    surv1<-data.frame(Prediction=unlist(pred_surv1$P1[,"pred_CIF"]),UL=unlist(pred_surv1$P1[,'upp_CIF']),LL=unlist(pred_surv1$P1[,'low_CIF']),Time=unlist(pred_surv1$P1[,timeVar])),
                    surv2<-data.frame(Prediction=unlist(pred_surv2$P1[,"pred_CIF"]),UL=unlist(pred_surv2$P1[,'upp_CIF']),LL=unlist(pred_surv2$P1[,'low_CIF']),Time=unlist(pred_surv2$P1[,timeVar]))
    )
    pred_list[[3]]$Prediction<-1-pred_list[[3]]$Prediction
    pred_list[[3]]$UL<-1-pred_list[[3]]$UL
    pred_list[[3]]$LL<-1-pred_list[[3]]$LL
    pred_list[[4]]$Prediction<-1-pred_list[[4]]$Prediction
    pred_list[[4]]$UL<-1-pred_list[[4]]$UL
    pred_list[[4]]$LL<-1-pred_list[[4]]$LL
  }
  oldpar <- par(no.readonly = TRUE)
  K<-2
  m<-cbind(1:K,rep(K+1,K),rep(K+2,K))
  xticks<-pretty(c(pred_list[[1]]$Time,pred_list[[2]]$Time))
  widths<-c(0.3,0.35,0.35)
  layout(m,widths=widths)
  par(mar=c(0,4.5,0,0),oma=c(4,0,3,0),cex.axis=1.1,font.axis=2,font.lab=2,font.main=2)
  plot(x=pred_list[[1]]$Time,y=pred_list[[1]]$yvalue,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=pred_list[[1]]$Time,y=pred_list[[1]]$yvalue,pch=8,col='red')
  box(lwd=1.3)
  mtext(long1_res,line=2.5,side=2,font=2)
  par(mar = c(0, 4.5, 0, 0))
  plot(x=pred_list[[2]]$Time,y=pred_list[[2]]$yvalue,type='l',las=1,ylab='',xaxt='n',lwd=1.5)
  points(x=pred_list[[2]]$Time,y=pred_list[[2]]$yvalue,pch=8,col='red')
  axis(1, at = xticks)
  box(lwd=1.3)
  mtext(long2_res,line=2.5,side=2,font=2)
  xticks<-pretty(c(pred_list[[3]]$Time))
  par(mar=c(0,0,0,0))
  plot(x=pred_list[[3]]$Time,y=pred_list[[3]]$Prediction,col='black',type='l',las=1,yaxt='n',xaxs='i',xaxt='n',ylab='',ylim=c(0,1))
  polygon(c(pred_list[[3]]$Time,rev(pred_list[[3]]$Time)),c(pred_list[[3]]$UL,rev(pred_list[[3]]$LL)),col=rgb(0,0,0.6,0.4),border=NA)
  lines(x=pred_list[[3]]$Time,y=pred_list[[3]]$Prediction,col='blue',lwd=2)
  text(x=median(pred_list[[3]]$Time),y=0.99,labels =ifelse(isTRUE(survtime[1]<survtime[2]),'Event 1','Event 2'), pos = 3, col = "black",cex=1.6,font=2)
  axis(1, at = xticks)
  box(lwd=1.3)
  xticks<-pretty(c(pred_list[[4]]$Time))
  par(mar=c(0,0,0,4.5))
  plot(x=pred_list[[4]]$Time,y=pred_list[[4]]$Prediction,col='black',type='l',las=1,yaxt='n',xaxs='i',xaxt='n',ylab='',ylim=c(0,1))
  polygon(c(pred_list[[4]]$Time,rev(pred_list[[4]]$Time)),c(pred_list[[4]]$UL,rev(pred_list[[4]]$LL)),col=rgb(0.6,0,0.6,0.4),border = NA)
  lines(x=pred_list[[4]]$Time,pred_list[[4]]$Prediction,col='red',lwd=2)
  text(x=median(pred_list[[4]]$Time),y=0.99,labels = ifelse(isTRUE(survtime[1]<survtime[2]),'Event 2','Event 1'), pos = 3, col = "black",cex=1.6,font=2)
  axis(1, at = xticks)
  axis(4,las=1)
  mtext('Plot for Bidirectional survival data', 3,
        line = 1, outer = TRUE, font = 2, cex = 1.3)
  mtext("Time", 1,
        line = 2.5, outer = TRUE,font=2)
  mtext("Survival Prediction", 4,
        line = 2.5,font=2)
  box(lwd=1.3)
  on.exit(par(oldpar))
}

utils::globalVariables(c('predict'))

#' @title survival data
#' @docType data
#' @description A survival dataset related the long2 dataset, with different numeric and categorical covariate
#' @usage data(new_surv2)
#' @format a tibble of 13 columns and 1000 observations,
#' \describe{
#' \item{id}{id value for subjects}
#' \item{status}{survival status}
#' \item{time}{ survival time}
#' \item{visit}{visit time of longitudinal measurements}
#' \item{x1,x2,...,x7}{ different numeric and categorical variable}
#'
#' }
#'
"new_surv2"
#' @title longitudinal- survival dataset
#' @docType  data
#' @description A longitudinal dataset with single marker , with different numeric and categorical covariate
#' @usage data(new_long2)
#' @format a tibble of 13 columns and 5639 observations,
#' \describe{
#' \item{id}{id value for subjects}
#' \item{status}{survival status}
#' \item{time}{ survival time}
#' \item{y}{longitudinal marker}
#' \item{visit}{visit time of longitudinal measurements}
#' \item{x1,x2,...,x7}{ different numeric and categorical variable}
#'
#' }
#'
"new_long2"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.jmstB}
\alias{predict.jmstB}
\title{predict.jmstB}
\usage{
\method{predict}{jmstB}(object, newdata, ...)
}
\arguments{
\item{object}{fitted model}

\item{newdata}{newdata}

\item{...}{others}
}
\value{
Survival Prediction for newdata from model fitted through
\code{jmstB()}
}
\description{
predict.jmstB
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset.R
\docType{data}
\name{new_surv2}
\alias{new_surv2}
\title{survival data}
\format{
a tibble of 13 columns and 1000 observations,
\describe{
\item{id}{id value for subjects}
\item{status}{survival status}
\item{time}{ survival time}
\item{visit}{visit time of longitudinal measurements}
\item{x1,x2,...,x7}{ different numeric and categorical variable}

}
}
\usage{
data(new_surv2)
}
\description{
A survival dataset related the long2 dataset, with different numeric and categorical covariate
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.jmbB.R
\name{plot.jmbB}
\alias{plot.jmbB}
\title{Prediction plot from \code{jmbB()}}
\usage{
\method{plot}{jmbB}(x, y, ...)
}
\arguments{
\item{x}{fitted model}

\item{y}{newdata}

\item{...}{others}
}
\value{
Returns prediction plot for the newdata using the model fitted through \code{jmbB()}.
}
\description{
Prediction plot from \code{jmbB()}
}
\note{
In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
8.832. Now suppose we want to look at the survival of ID 2 under joint model
1 after time 4 and for joint model 2 after time 9. For that we created the
newdata as if the individual is followed till for a time period
less than min(4,9).
}
\examples{
 \donttest{
##
library(JMbayes2)
st_pbcid<-function(){
  new_pbcid<-pbc2.id
  new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
  cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
  status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
  new_pbcid$status_2<-status_2
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
  new_pbcid
}
new_pbc2id<-st_pbcid()
pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
pbc2_new<-pbc2[pbc2$id\%in\%c(1:100),]
new_pbc2id<-new_pbc2id[new_pbc2id$id\%in\%c(1:100),]
model_jmbBdirect<-jmbB(dtlong=pbc2_new,dtsurv = new_pbc2id,
                       longm=list(serBilir ~ drug*year,serBilir ~ drug*year),
                       survm=list(Surv(years,status2) ~ drug,
                                  Surv(time_2,status_2) ~ drug+age),
                       rd=list(~year|id,~year|id),
                       id='id',timeVar ='year')

t0<-4
nd <- pbc2[pbc2$id \%in\% c(2), ]
nd<-nd[nd$year<t0,]
nd$status2<-0
nd$years<-t0
nd$time_2<-9
nd$status_2<-0
plot(model_jmbBdirect,nd)
nd <- pbc2[pbc2$id \%in\% c(2), ]
nd<-nd[nd$year<12,]
nd$status2<-0
nd$years<-12
nd$time_2<-9
nd$status_2<-0
plot(model_jmbBdirect,nd)
##
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jmrmlB.R
\name{jmrmlB}
\alias{jmrmlB}
\title{Joint model for Bidirectional survival data using \code{joineRML}}
\usage{
jmrmlB(
  dtlong,
  dtsurv,
  longm,
  survm,
  rd,
  timeVar,
  id,
  samplesize = 200,
  BIGdata = FALSE
)
}
\arguments{
\item{dtlong}{longitudinal data}

\item{dtsurv}{survival data with two event status along with event time}

\item{longm}{longitudinal model e.g. list(serBilir~drug * year,serBilir ~ drug * year)}

\item{survm}{survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)}

\item{rd}{random effect component e.g. list(~year|id,~year|id)}

\item{timeVar}{time variable}

\item{id}{ID variable}

\item{samplesize}{samplesize for bigdata}

\item{BIGdata}{logical argument TRUE or FALSE}
}
\value{
Estimated model parameters of Joint model with bidirectional survival data
}
\description{
The function fits joint model for survival data with two events. It utilizes the joineRML package for obtaining the model parameter estimates.
}
\examples{
 \donttest{
##
library(JMbayes2)
library(joineRML)
jmrmlBModel<-jmrmlB(dtlong=new_long2[new_long2$id\%in\%c(1:80),],
                    dtsurv=new_surv2[new_surv2$id\%in\%c(1:80),],
                    longm=list(y~x7+visit,y~x7+visit),survm=list(Surv(time,status)~x1+visit,
                    Surv(time_2,status_2)~x1+visit),rd=list(~visit|id,~visit|id),id='id',
                    timeVar='visit',samplesize=40,BIGdata=TRUE)
jmrmlBModel
##
}
}
\references{
Hickey, Graeme L., et al. "joineRML: a joint model and software package for time-to-event and multivariate longitudinal outcomes." BMC medical research methodology 18 (2018): 1-14.

Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
}
\author{
Atanu Bhattacharjee
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jmbB.R
\name{jmbB}
\alias{jmbB}
\title{Joint model for Bidirectional survival data using \code{JMbayes2}}
\usage{
jmbB(
  dtlong,
  dtsurv,
  longm,
  survm,
  rd,
  timeVar,
  id,
  samplesize = NULL,
  BIGdata = FALSE,
  niter = 200,
  nburnin = 100,
  nchain = 1
)
}
\arguments{
\item{dtlong}{longitudinal data}

\item{dtsurv}{survival data with two event status along with event time}

\item{longm}{longitudinal model e.g. list(serBilir~drug * year,serBilir ~ drug * year)}

\item{survm}{survival model e.g. list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age)}

\item{rd}{random effect component e.g. list(~year|id,~year|id)}

\item{timeVar}{time variable}

\item{id}{ID variable}

\item{samplesize}{samplesize for bigdata}

\item{BIGdata}{logical argument TRUE or FALSE}

\item{niter}{number if iteration}

\item{nburnin}{number of sample to burn}

\item{nchain}{number of MCMC chain}
}
\value{
Estimated model parameters of Joint model with bidirectional survival data
}
\description{
The function fits joint model for survival data with two events. It utilizes the JMbayes2 package for obtaining the model parameter estimates.
}
\examples{
library(JMbayes2)
st_pbcid<-function(){
  new_pbcid<-pbc2.id
  new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
  cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
  status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
  new_pbcid$status_2<-status_2
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
  new_pbcid
}
new_pbc2id<-st_pbcid()
pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
pbc2_new<-pbc2[pbc2$id\%in\%c(1:100),]
new_pbc2id<-new_pbc2id[new_pbc2id$id\%in\%c(1:100),]
model_jmbBdirect<-jmbB(dtlong=pbc2_new,dtsurv =new_pbc2id,
                       longm=list(serBilir~drug*year,serBilir~drug*year),
                       survm=list(Surv(years,status2)~drug,Surv(time_2,status_2)~drug+age),
                       rd=list(~year|id,~year|id),
                       id='id',timeVar ='year')
model_jmbBdirect
}
\references{
Rizopoulos, D., G. Papageorgiou, and P. Miranda Afonso. "JMbayes2: extended joint models for longitudinal and time-to-event data." R package version 0.2-4 (2022).

Bhattacharjee, A., Rajbongshi, B. K., & Vishwakarma, G. K. (2024). jmBIG: enhancing dynamic risk prediction and personalized medicine through joint modeling of longitudinal and survival data in big routinely collected data. BMC Medical Research Methodology, 24(1), 172.
}
\author{
Atanu Bhattacharjee
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootciJMCS.R
\name{bootciJMCS}
\alias{bootciJMCS}
\title{Function for bootstrapped confidence interval}
\usage{
bootciJMCS(object, future_time)
}
\arguments{
\item{object}{fitted model}

\item{future_time}{time sequence at which estimates are required}
}
\value{
Returns bootstraped confidence interval for model fitted through FastJM
}
\description{
Function for bootstrapped confidence interval
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.jmrmlB.R
\name{plot.jmrmlB}
\alias{plot.jmrmlB}
\title{Prediction plot from \code{jmrmlB()}}
\usage{
\method{plot}{jmrmlB}(x, y, ...)
}
\arguments{
\item{x}{fitted model object}

\item{y}{newdata}

\item{...}{others}
}
\value{
Returns prediction plot for the newdata using the model fitted through \code{jmrmlB()}.
}
\description{
Prediction plot from \code{jmrmlB()}
}
\note{
In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
8.832. Now suppose we want to look at the survival of ID 2 under joint model
1 after time 4 and for joint model 2 after time 9. For that we created the
newdata as if the individual is followed till for a time period
less than min(4,9).
}
\examples{
 \donttest{
##
library(JMbayes2)
library(joineRML)
jmrmlBModel<-jmrmlB(dtlong=new_long2[new_long2$id<=400,],
                    dtsurv=new_surv2[new_surv2$id<=400,],
                    longm=list(y~x7+visit,y~x7+visit),
                    survm=list(Surv(time,status)~x1+visit,
                               Surv(time_2,status_2)~x1+visit),
                    rd=list(~visit|id,~visit|id),
                    id='id',
                    timeVar='visit',
                    samplesize=200,
                    BIGdata=TRUE)
t0<-6
ndBIG<-new_long2[new_long2$id==10,]
ndBIG<-ndBIG[ndBIG$visit<t0,]
ndBIG$status<-0
ndBIG$time<-t0
ndBIG$time_2<-10
ndBIG$status_2<-0
plot(jmrmlBModel,ndBIG)
##
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.jmcsB}
\alias{print.jmcsB}
\title{print.}
\usage{
\method{print}{jmcsB}(x, ...)
}
\arguments{
\item{x}{fittedobject}

\item{...}{others}
}
\value{
prints table containing various parameter estimates,
        SE, P- value for both survival and longitudinal submodel,
        if the model is bayesian it includes their credible interval too.
}
\description{
print method for class 'jmcsB'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.jmstB.R
\name{plot.jmstB}
\alias{plot.jmstB}
\title{Prediction plot from \code{jmstB()}}
\usage{
\method{plot}{jmstB}(x, y, ...)
}
\arguments{
\item{x}{fitted model object}

\item{y}{newdata}

\item{...}{others}
}
\value{
Returns prediction plot for the newdata using the model fitted through \code{jmstB()}.
}
\description{
Prediction plot from \code{jmstB()}
}
\note{
In the example code we use newdata as the data for ID 2 in the PBC2 dataset, it has follow up information till
8.832. Now suppose we want to look at the survival of ID 2 under joint model
1 after time 4 and for joint model 2 after time 9. For that we created the
newdata as if the individual is followed till for a time period
less than min(4,9).
}
\examples{
 \donttest{
##
library(JMbayes2)
library(rstanarm)
st_pbcid<-function(){
  new_pbcid<-pbc2.id
  new_pbcid$time_2<-rexp(n=nrow(pbc2.id),1/10)
  cen_time<-runif(nrow(pbc2.id),min(new_pbcid$time_2),max(new_pbcid$time_2))
  status_2<-ifelse(new_pbcid$time_2<cen_time,1,0)
  new_pbcid$status_2<-status_2
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<cen_time,new_pbcid$time_2,cen_time)
  new_pbcid$time_2<-ifelse(new_pbcid$time_2<new_pbcid$years,new_pbcid$years,new_pbcid$time_2)
  new_pbcid
}
new_pbc2id<-st_pbcid()
pbc2$status_2<-rep(new_pbc2id$status_2,times=data.frame(table(pbc2$id))$Freq)
pbc2$time_2<-rep(new_pbc2id$time_2,times=data.frame(table(pbc2$id))$Freq)
pbc2_new<-pbc2[pbc2$id\%in\%c(1:50),]
new_pbc2id<-new_pbc2id[new_pbc2id$id\%in\%c(1:50),]
model_jmstBdirect<-jmstB(
  dtlong=pbc2_new,
  dtsurv = new_pbc2id,
  longm=list(serBilir ~ drug * year+(year|id),albumin~drug+year+(year|id)),
  survm=list(Surv(years,status2) ~ drug,Surv(time_2,status_2) ~ drug),
  timeVar="year",
  id='id',
  refresh=400,
  nchain=1)
t0<-4
nd<-pbc2[pbc2$id \%in\% c(2), ]
nd<-nd[nd$year<t0,]
nd$status2<-0
nd$years<-t0
nd$time_2<-9
nd$status_2<-0
plot(x=model_jmstBdirect,y = nd)
##
}
}
